@@ -15,13 +15,14 @@ from __future__ import absolute_import, division, print_function
 
 import re
 import os.path
+from collections import OrderedDict
 
 
 ctype_to_dtype = {'double'     : "numpy.double",
                   'int'        : "numpy.intc",
                   'eraASTROM'  : "dt_eraASTROM",
                   'eraLDBODY'  : "dt_eraLDBODY",
-                  'char'       : "numpy.dtype('S16')",
+                  'char'       : "numpy.dtype('S1')",
                   'const char' : "numpy.dtype('S16')",
                   }
 
@@ -181,6 +182,17 @@ class Argument(object):
     def ndim(self):
         return len(self.shape)
 
+    @property
+    def cshape(self):
+        return ''.join(['[{0}]'.format(s) for s in self.shape])
+
+    @property
+    def name_for_call(self):
+        if self.is_ptr:
+            return '_'+self.name
+        else:
+            return '*_'+self.name
+
     def __repr__(self):
         return "Argument('{0}', name='{1}', ctype='{2}', inout_state='{3}')".format(self.definition, self.name, self.ctype, self.inout_state)
 
@@ -378,12 +390,14 @@ def main(srcdir, outfn, templateloc, verbose=True):
     with open(erfahfn, "r") as f:
         erfa_h = f.read()
 
-    funcs = {}
+    funcs = OrderedDict()
     section_subsection_functions = re.findall('/\* (\w*)/(\w*) \*/\n(.*?)\n\n',
                                               erfa_h, flags=re.DOTALL|re.MULTILINE)
     for section, subsection, functions in section_subsection_functions:
         print_("{0}.{1}".format(section, subsection))
-        if section == "Astronomy":
+        if ((section == "Astronomy") or (subsection == "AngleOps")
+            or (subsection == "SphericalCartesian")
+            or (subsection == "MatrixVectorProducts")):
             func_names = re.findall(' (\w+)\(.*?\);', functions, flags=re.DOTALL)
             for name in func_names:
                 print_("{0}.{1}.{2}...".format(section, subsection, name))
