@@ -12,11 +12,13 @@ from ..extern.six.moves import zip
 from ..units import Unit, IrreducibleUnit
 from .. import units as u
 from ..wcs.utils import skycoord_to_pixel, pixel_to_skycoord
-from ..utils.exceptions import AstropyDeprecationWarning
+from ..utils.exceptions import AstropyDeprecationWarning, AstropyWarning
+from ..utils.data_info import MixinInfo
 
 from .distances import Distance
+from .angles import Angle
 from .baseframe import BaseCoordinateFrame, frame_transform_graph, GenericFrame, _get_repr_cls
-from .builtin_frames import ICRS
+from .builtin_frames import ICRS, SkyOffsetFrame
 from .representation import (BaseRepresentation, SphericalRepresentation,
                              UnitSphericalRepresentation)
 
@@ -41,6 +43,43 @@ def FRAME_ATTR_NAMES_SET():
     return out
 
 
+class SkyCoordInfo(MixinInfo):
+    """
+    Container for meta information like name, description, format.  This is
+    required when the object is used as a mixin column within a table, but can
+    be used as a general way to store meta information.
+    """
+    attrs_from_parent = set(['unit'])  # Unit is read-only
+    _supports_indexing = False
+
+    @staticmethod
+    def default_format(val):
+        repr_data = val.info._repr_data
+        formats = ['{0.' + compname + '.value:}' for compname
+                   in repr_data.components]
+        return ','.join(formats).format(repr_data)
+
+    @property
+    def unit(self):
+        repr_data = self._repr_data
+        unit = ','.join(str(getattr(repr_data, comp).unit) or 'None'
+                        for comp in repr_data.components)
+        return unit
+
+    @property
+    def _repr_data(self):
+        if self._parent is None:
+            return None
+
+        sc = self._parent
+        if (issubclass(sc.representation, SphericalRepresentation) and
+                isinstance(sc.data, UnitSphericalRepresentation)):
+            repr_data = sc.represent_as(sc.data.__class__, in_frame_units=True)
+        else:
+            repr_data = sc.represent_as(sc.representation, in_frame_units=True)
+        return repr_data
+
+
 class SkyCoord(object):
     """High-level object providing a flexible interface for celestial coordinate
     representation, manipulation, and transformation between systems.
@@ -121,6 +160,9 @@ class SkyCoord(object):
         Specifies the representation, e.g. 'spherical', 'cartesian', or
         'cylindrical'.  This affects the positional args and other keyword args
         which must correspond to the given representation.
+    copy: bool, optional
+        If `True` (default), a copy of any coordinate data is made.  This
+        argument can only be passed in as a keyword argument.
     **keyword_args
         Other keyword arguments as applicable for user-defined coordinate frames.
         Common options include:
@@ -139,10 +181,10 @@ class SkyCoord(object):
             Cartesian coordinates values for the Galactic frame.
     """
 
-
     # Declare that SkyCoord can be used as a Table column by defining the
-    # attribute where column attributes will be stored.
-    _astropy_column_attrs = None
+    # info property.
+    info = SkyCoordInfo()
+
 
     def __init__(self, *args, **kwargs):
 
@@ -150,6 +192,7 @@ class SkyCoord(object):
         # kwargs dict for initializing attributes for this object and for
         # creating the internal self._sky_coord_frame object
         args = list(args)  # Make it mutable
+        copy = kwargs.pop('copy', True)
         kwargs = self._parse_inputs(args, kwargs)
 
         # Set internal versions of object state attributes
@@ -166,7 +209,7 @@ class SkyCoord(object):
                 coord_kwargs[attr] = value
 
         # Finally make the internal coordinate object.
-        self._sky_coord_frame = frame.__class__(**coord_kwargs)
+        self._sky_coord_frame = frame.__class__(copy=copy, **coord_kwargs)
 
         if not self._sky_coord_frame.has_data:
             raise ValueError('Cannot create a SkyCoord without data')
@@ -198,7 +241,15 @@ class SkyCoord(object):
             # First turn `self` into a mockup of the thing we want - we can copy
             # this to get all the right attributes
             self._sky_coord_frame = self_frame[item]
-            return SkyCoord(self, representation=self.representation)
+            out = SkyCoord(self, representation=self.representation)
+
+            # Copy other 'info' attr only if it has actually been defined.
+            # See PR #3898 for further explanation and justification, along
+            # with Quantity.__array_finalize__
+            if 'info' in self.__dict__:
+                out.info = self.info
+
+            return out
         finally:
             # now put back the right frame in self
             self._sky_coord_frame = self_frame
@@ -244,6 +295,10 @@ class SkyCoord(object):
                 # were explicitly specified in the coordinate object (i.e. non-default).
                 coord_kwargs = _parse_coordinate_arg(args[0], frame, units, kwargs)
 
+                # Copy other 'info' attr only if it has actually been defined.
+                if 'info' in getattr(args[0], '__dict__', ()):
+                    self.info = args[0].info
+
             elif len(args) <= 3:
                 frame_attr_names = frame.representation_component_names.keys()
                 repr_attr_names = frame.representation_component_names.values()
@@ -369,7 +424,17 @@ class SkyCoord(object):
                 if attr in self.frame.get_frame_attr_names():
                     return getattr(self.frame, attr)
                 else:
-                    return getattr(self, '_' + attr)
+                    try:
+                        return getattr(self, '_' + attr)
+                    except AttributeError:
+                        # this can happen because FRAME_ATTR_NAMES_SET is
+                        # dynamic.  So if a frame is added to the transform
+                        # graph after this SkyCoord was created, the "real"
+                        # underlying attribute - e.g. `_equinox` does not exist
+                        # on the SkyCoord.  So we add this case to just use
+                        # None, as it wouldn't have been possible for the user
+                        # to have set the value until the frame existed anyway
+                        return None
 
             # Some attributes might not fall in the above category but still
             # are available through self._sky_coord_frame.
@@ -624,10 +689,10 @@ class SkyCoord(object):
             raise TypeError('Can only get separation to another SkyCoord or a '
                             'coordinate frame with data')
 
-        if self.data.__class__ == UnitSphericalRepresentation:
+        if issubclass(self.data.__class__, UnitSphericalRepresentation):
             raise ValueError('This object does not have a distance; cannot '
                              'compute 3d separation.')
-        if other.data.__class__ == UnitSphericalRepresentation:
+        if issubclass(other.data.__class__, UnitSphericalRepresentation):
             raise ValueError('The other object does not have a distance; '
                              'cannot compute 3d separation.')
 
@@ -638,6 +703,53 @@ class SkyCoord(object):
         distval = (dx.value ** 2 + dy.value ** 2 + dz.value ** 2) ** 0.5
         return Distance(distval, dx.unit)
 
+    def spherical_offsets_to(self, tocoord):
+        r"""
+        Computes angular offsets to go *from* this coordinate *to* another.
+
+        Parameters
+        ----------
+        tocoord : `~astropy.coordinates.BaseCoordinateFrame`
+            The coordinate to offset to.
+
+        Returns
+        -------
+        lon_offset : `~astropy.coordinates.Angle`
+            The angular offset in the longitude direction (i.e., RA for
+            equatorial coordinates).
+        lat_offset : `~astropy.coordinates.Angle`
+            The angular offset in the latitude direction (i.e., Dec for
+            equatorial coordinates).
+
+        Raises
+        ------
+        ValueError
+            If the ``tocoord`` is not in the same frame as this one. This is
+            different from the behavior of the `separation`/`separation_3d`
+            methods because the offset components depend critically on the
+            specific choice of frame.
+
+        Notes
+        -----
+        This uses the sky offset frame machinery, and hence will produce a new
+        sky offset frame if one does not already exist for this object's frame
+        class.
+
+        See Also
+        --------
+        separation : for the *total* angular offset (not broken out into components)
+
+        """
+        if not self.is_equivalent_frame(tocoord):
+            raise ValueError('Tried to use spherical_offsets_to with two non-matching frames!')
+
+        aframe = self.skyoffset_frame()
+        acoord = tocoord.transform_to(aframe)
+
+        dlon = acoord.spherical.lon.view(Angle)
+        dlat = acoord.spherical.lat.view(Angle)
+        return dlon, dlat
+
     def match_to_catalog_sky(self, catalogcoord, nthneighbor=1):
         """
         Finds the nearest on-sky matches of this coordinate in a set of
@@ -925,6 +1037,60 @@ class SkyCoord(object):
 
         return angle_utilities.position_angle(slon, slat, olon, olat)
 
+    def skyoffset_frame(self, rotation=None):
+        """
+        Returns the sky offset frame with this `SkyCoord` at the origin.
+
+        Returns
+        -------
+        astrframe : `~astropy.coordinates.SkyOffsetFrame`
+            A sky offset frame of the same type as this `SkyCoord` (e.g., if
+            this object has an ICRS coordinate, the resulting frame is
+            SkyOffsetICRS, with the origin set to this object)
+        rotation : `~astropy.coordinates.Angle` or `~astropy.units.Quantity` with angle units
+            The final rotation of the frame about the ``origin``. The sign of
+            the rotation is the left-hand rule. That is, an object at a
+            particular position angle in the un-rotated system will be sent to
+            the positive latitude (z) direction in the final frame.
+        """
+        return SkyOffsetFrame(origin=self, rotation=rotation)
+
+    def get_constellation(self, short_name=False, constellation_list='iau'):
+        """
+        Determines the constellation(s) of the coordinates this `SkyCoord`
+        contains.
+
+        Parameters
+        ----------
+        short_name : bool
+            If True, the returned names are the IAU-sanctioned abbreviated
+            names.  Otherwise, full names for the constellations are used.
+        constellation_list : str
+            The set of constellations to use.  Currently only ``'iau'`` is
+            supported, meaning the 88 "modern" constellations endorsed by the IAU.
+
+        Returns
+        -------
+        constellation : str or string array
+            If this is a scalar coordinate, returns the name of the
+            constellation.  If it is an array `SkyCoord`, it returns an array of
+            names.
+
+        Notes
+        -----
+        To determine which constellation a point on the sky is in, this first
+        precesses to B1875, and then uses the Delporte boundaries of the 88
+        modern constellations, as tabulated by
+        `Roman 1987 <http://cdsarc.u-strasbg.fr/viz-bin/Cat?VI/42>`_.
+
+        See Also
+        --------
+        astropy.coordinates.get_constellation
+        """
+        from .funcs import get_constellation
+
+        return get_constellation(self, short_name, constellation_list)
+
     # WCS pixel to/from sky conversions
     def to_pixel(self, wcs, origin=0, mode='all'):
         """
@@ -948,7 +1114,6 @@ class SkyCoord(object):
 
         See Also
         --------
-        from_pixel : to do the inverse operation
         astropy.wcs.utils.skycoord_to_pixel : the implementation of this method
         """
         return skycoord_to_pixel(self, wcs=wcs, origin=origin, mode=mode)
@@ -1321,8 +1486,8 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
             allunitsphrepr = True
             for sc in scs[1:]:
                 if not sc.is_equivalent_frame(scs[0]):
-                        raise ValueError("List of inputs don't have equivalent "
-                                         "frames: {0} != {1}".format(sc, scs[0]))
+                    raise ValueError("List of inputs don't have equivalent "
+                                     "frames: {0} != {1}".format(sc, scs[0]))
                 if allunitsphrepr and not isinstance(sc.data, UnitSphericalRepresentation):
                     allunitsphrepr = False
 
@@ -1395,7 +1560,8 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
     try:
         for frame_attr_name, repr_attr_class, value, unit in zip(
                 frame_attr_names, repr_attr_classes, values, units):
-            valid_kwargs[frame_attr_name] = repr_attr_class(value, unit=unit)
+            valid_kwargs[frame_attr_name] = repr_attr_class(value, unit=unit,
+                                                            copy=False)
     except Exception as err:
         raise ValueError('Cannot parse first argument data "{0}" for attribute '
                          '{1}'.format(value, frame_attr_name), err)
