@@ -8,6 +8,7 @@ from __future__ import (absolute_import, division, print_function,
 
 """Test initialization of angles not already covered by the API tests"""
 
+import pickle
 import numpy as np
 
 from ..earth import EarthLocation, ELLIPSOIDS
@@ -199,7 +200,7 @@ class TestInput():
         assert loc_slice1.unit is locwgs72.unit
         assert loc_slice1.ellipsoid == locwgs72.ellipsoid == 'WGS72'
         assert not loc_slice1.shape
-        with pytest.raises(IndexError):
+        with pytest.raises(TypeError):
             loc_slice1[0]
         with pytest.raises(IndexError):
             len(loc_slice1)
@@ -248,3 +249,20 @@ class TestInput():
             assert allclose_m14(location.z.value, self.z.value)
         else:
             assert not np.all(isclose_m14(location.z.value, self.z.value))
+
+
+def test_pickling():
+    """Regression test against #4304."""
+    el = EarthLocation(0.*u.m, 6000*u.km, 6000*u.km)
+    s = pickle.dumps(el)
+    el2 = pickle.loads(s)
+    assert el == el2
+
+def test_repr_latex():
+    """
+    Regression test for issue #4542
+    """
+    somelocation = EarthLocation(lon='149:3:57.9', lat='-31:16:37.3')
+    somelocation._repr_latex_()
+    somelocation2 = EarthLocation(lon=[1., 2.]*u.deg, lat=[-1., 9.]*u.deg)
+    somelocation2._repr_latex_()
