@@ -8,36 +8,18 @@ cds.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-##
-## Redistribution and use in source and binary forms, with or without
-## modification, are permitted provided that the following conditions are met:
-##     * Redistributions of source code must retain the above copyright
-##       notice, this list of conditions and the following disclaimer.
-##     * Redistributions in binary form must reproduce the above copyright
-##       notice, this list of conditions and the following disclaimer in the
-##       documentation and/or other materials provided with the distribution.
-##     * Neither the name of the Smithsonian Astrophysical Observatory nor the
-##       names of its contributors may be used to endorse or promote products
-##       derived from this software without specific prior written permission.
-##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-## DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-## DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-## (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-## LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-## ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+from __future__ import absolute_import, division, print_function
 
 import fnmatch
 import itertools
 import re
+import os
 
 from . import core
 from . import fixedwidth
 
+from ...utils.compat import ignored
+
 
 __doctest_skip__ = ['*']
 
@@ -48,6 +30,9 @@ class CdsHeader(core.BaseHeader):
                     'i': core.IntType,
                     'a': core.StrType}
 
+    'The ReadMe file to construct header from.'
+    readme = None
+
     def get_type_map_key(self, col):
         match = re.match(r'\d*(\S)', col.raw_type.lower())
         if not match:
@@ -55,33 +40,25 @@ class CdsHeader(core.BaseHeader):
                 col.raw_type, col.name))
         return match.group(1)
 
-    def __init__(self, readme=None):
-        """Initialize ReadMe filename.
-
-        :param readme: The ReadMe file to construct header from.
-        :type readme: String
-
-        CDS tables have their header information in a separate file
-        named "ReadMe". The ``get_cols`` method will read the contents
-        of the ReadMe file given by ``self.readme`` and set the various
-        properties needed to read the data file. The data file name
-        will be the ``table`` passed to the ``read`` method.
-        """
-        core.BaseHeader.__init__(self)
-        self.readme = readme
 
     def get_cols(self, lines):
-        """Initialize the header Column objects from the table ``lines`` for a CDS
+        """
+        Initialize the header Column objects from the table ``lines`` for a CDS
         header.
 
-        :param lines: list of table lines
-        :returns: list of table Columns
+        Parameters
+        ----------
+        lines : list
+            List of table lines
+
         """
+
         # Read header block for the table ``self.data.table_name`` from the read
         # me file ``self.readme``.
         if self.readme and self.data.table_name:
             in_header = False
-            f = open(self.readme, "r")
+            readme_inputter = core.BaseInputter()
+            f = readme_inputter.get_lines(self.readme)
             # Header info is not in data lines but in a separate file.
             lines = []
             comment_lines = 0
@@ -111,7 +88,6 @@ class CdsHeader(core.BaseHeader):
             else:
                 raise core.InconsistentTableError("Can't find table {0} in {1}".format(
                     self.data.table_name, self.readme))
-            f.close()
 
         found_line = False
 
@@ -127,8 +103,8 @@ class CdsHeader(core.BaseHeader):
                                     (?P<end>   \d+)        \s+
                                     (?P<format> [\w.]+)     \s+
                                     (?P<units> \S+)        \s+
-                                    (?P<name>  \S+)        \s+
-                                    (?P<descr> \S.+)""",
+                                    (?P<name>  \S+)
+                                    (\s+ (?P<descr> \S.*))?""",
                                 re.VERBOSE)
 
         cols = []
@@ -144,18 +120,19 @@ class CdsHeader(core.BaseHeader):
                 col.unit = match.group('units')
                 if col.unit == '---':
                     col.unit = None  # "---" is the marker for no unit in CDS table
-                col.description = match.group('descr').strip()
+                col.description = (match.group('descr') or '').strip()
                 col.raw_type = match.group('format')
                 col.type = self.get_col_type(col)
 
                 match = re.match(
-                    r'\? (?P<equal> =)? (?P<nullval> \S*)', col.description, re.VERBOSE)
+                    r'\? (?P<equal> =)? (?P<nullval> \S*) (\s+ (?P<descriptiontext> \S.*))?', col.description, re.VERBOSE)
                 if match:
+                    col.description=(match.group('descriptiontext') or '').strip()
                     if issubclass(col.type, core.FloatType):
                         fillval = 'nan'
                     else:
                         fillval = '0'
-                    
+
                     if match.group('nullval') == '-':
                         col.null = '---'
                         # CDS tables can use -, --, ---, or ---- to mark missing values
@@ -253,6 +230,20 @@ class Cds(core.BaseReader):
       >>> table = ascii.read("t/cds/multi/lhs2065.dat", readme="t/cds/multi/ReadMe")
       >>> table = ascii.read("t/cds/glob/lmxbrefs.dat", readme="t/cds/glob/ReadMe")
 
+    The table name and the CDS ReadMe file can be entered as URLs.  This can be used
+    to directly load tables from the Internet.  For example, Vizier tables from the
+    CDS::
+
+      >>> table = ascii.read("ftp://cdsarc.u-strasbg.fr/pub/cats/VII/253/snrs.dat",
+      ...             readme="ftp://cdsarc.u-strasbg.fr/pub/cats/VII/253/ReadMe")
+
+    If the header (ReadMe) and data are stored in a single file and there
+    is content between the header and the data (for instance Notes), then the
+    parsing process may fail.  In this case you can instruct the reader to
+    guess the actual start of the data by supplying ``data_start='guess'`` in the
+    call to the ``ascii.read()`` function.  You should verify that the output
+    data table matches expectation based on the input CDS file.
+
     **Using a reader object**
 
     When ``Cds`` reader object is created with a ``readme`` parameter
@@ -291,11 +282,41 @@ class Cds(core.BaseReader):
     _io_registry_can_write = False
     _description = 'CDS format table'
 
+    data_class = CdsData
+    header_class = CdsHeader
+
     def __init__(self, readme=None):
-        core.BaseReader.__init__(self)
-        self.header = CdsHeader(readme)
-        self.data = CdsData()
+        super(Cds, self).__init__()
+        self.header.readme = readme
 
     def write(self, table=None):
         """Not available for the Cds class (raises NotImplementedError)"""
         raise NotImplementedError
+
+    def read(self, table):
+        # If the read kwarg `data_start` is 'guess' then the table may have extraneous
+        # lines between the end of the header and the beginning of data.
+        if self.data.start_line == 'guess':
+            # Replicate the first part of BaseReader.read up to the point where
+            # the table lines are initially read in.
+            with ignored(TypeError):
+                # For strings only
+                if os.linesep not in table + '':
+                    self.data.table_name = os.path.basename(table)
+
+            self.data.header = self.header
+            self.header.data = self.data
+
+            # Get a list of the lines (rows) in the table
+            lines = self.inputter.get_lines(table)
+
+            # Now try increasing data.start_line by one until the table reads successfully.
+            # For efficiency use the in-memory list of lines instead of `table`, which
+            # could be a file.
+            for data_start in range(len(lines)):
+                self.data.start_line = data_start
+                with ignored(Exception):
+                    table = super(Cds, self).read(lines)
+                    return table
+        else:
+            return super(Cds, self).read(table)
