@@ -4,12 +4,13 @@
 
 import re
 from io import BytesIO
+from collections import OrderedDict
+import locale
 
 import numpy as np
+import platform
 
-from ....extern import six
 from ....extern.six.moves import cStringIO as StringIO
-from ....utils import OrderedDict
 from ....tests.helper import pytest
 from ... import ascii
 from ....table import Table
@@ -21,12 +22,19 @@ from .. import core
 from ..ui import _probably_html, get_read_trace
 
 try:
-    import bz2
+    import bz2  # pylint: disable=W0611
 except ImportError:
     HAS_BZ2 = False
 else:
     HAS_BZ2 = True
 
+try:
+    import pathlib
+except ImportError:
+    HAS_PATHLIB = False
+else:
+    HAS_PATHLIB = True
+
 
 @pytest.mark.parametrize('fast_reader', [True, False, 'force'])
 def test_convert_overflow(fast_reader):
@@ -791,7 +799,7 @@ def get_testfiles(name=None):
     ]
 
     try:
-        import bs4
+        import bs4  # pylint: disable=W0611
         testfiles.append({'cols': ('Column 1', 'Column 2', 'Column 3'),
                           'name': 't/html.html',
                           'nrows': 3,
@@ -858,6 +866,31 @@ def test_sextractor_units():
         assert table[colname].unit == expected_units[i]
         assert table[colname].description == expected_descrs[i]
 
+def test_sextractor_last_column_array():
+    """
+    Make sure that the SExtractor reader handles the last column correctly when it is array-like.
+    """
+    table = ascii.read('t/sextractor3.dat', Reader=ascii.SExtractor, guess=False)
+    expected_columns = ['X_IMAGE', 'Y_IMAGE', 'ALPHA_J2000', 'DELTA_J2000',
+                        'MAG_AUTO', 'MAGERR_AUTO',
+                        'MAG_APER', 'MAG_APER_1', 'MAG_APER_2', 'MAG_APER_3', 'MAG_APER_4', 'MAG_APER_5', 'MAG_APER_6',
+                        'MAGERR_APER', 'MAGERR_APER_1', 'MAGERR_APER_2', 'MAGERR_APER_3', 'MAGERR_APER_4', 'MAGERR_APER_5', 'MAGERR_APER_6']
+    expected_units = [Unit('pix'), Unit('pix'), Unit('deg'), Unit('deg'),
+                      Unit('mag'), Unit('mag'),
+                      Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'),
+                      Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag'), Unit('mag')]
+    expected_descrs = ['Object position along x', None,
+                       'Right ascension of barycenter (J2000)',
+                       'Declination of barycenter (J2000)',
+                       'Kron-like elliptical aperture magnitude',
+                       'RMS error for AUTO magnitude',] + [
+                       'Fixed aperture magnitude vector'] * 7 + [
+                       'RMS error vector for fixed aperture mag.'] * 7
+    for i, colname in enumerate(table.colnames):
+        assert table[colname].name == expected_columns[i]
+        assert table[colname].unit == expected_units[i]
+        assert table[colname].description == expected_descrs[i]
+
 def test_list_with_newlines():
     """
     Check that lists of strings where some strings consist of just a newline
@@ -1093,3 +1126,60 @@ def test_table_with_no_newline():
         t = ascii.read(table, **kwargs)
         assert t.colnames == ['a', 'b']
         assert len(t) == 0
+
+@pytest.mark.skipif('not HAS_PATHLIB')
+def test_path_object():
+    fpath = pathlib.Path('t/simple.txt')
+    data = ascii.read(fpath)
+
+    assert len(data) == 2
+    assert sorted(list(data.columns)) == ['test 1a', 'test2', 'test3', 'test4']
+    assert data['test2'][1] == 'hat2'
+
+
+def test_column_conversion_error():
+    """
+    Test that context information (upstream exception message) from column
+    conversion error is provided.
+    """
+    ipac = """\
+| col0   |
+| double |
+ 1  2
+"""
+    with pytest.raises(ValueError) as err:
+        ascii.read(ipac, guess=False, format='ipac')
+    assert 'Column col0 failed to convert:' in str(err.value)
+
+    with pytest.raises(ValueError) as err:
+        ascii.read(['a b', '1 2'], guess=False, format='basic', converters={'a': []})
+    assert 'no converters' in str(err.value)
+
+def test_non_C_locale_with_fast_reader():
+    """Test code that forces "C" locale while calling fast reader (#4364)"""
+    current = locale.setlocale(locale.LC_ALL)
+
+    try:
+        if platform.system() == 'Darwin':
+            locale.setlocale(locale.LC_ALL, str('de_DE'))
+        else:
+            locale.setlocale(locale.LC_ALL, str('de_DE.utf8'))
+
+        for fast_reader in (True, False, {'use_fast_converter': False}, {'use_fast_converter': True}):
+            t = ascii.read(['a b', '1.5 2'], format='basic', guess=False,
+                           fast_reader=fast_reader)
+            assert t['a'].dtype.kind == 'f'
+    except locale.Error as e:
+        pytest.skip('Locale error: {}'.format(e))
+    finally:
+        locale.setlocale(locale.LC_ALL, current)
+
+
+def test_no_units_for_char_columns():
+    '''Test that a char column of a Table is assigned no unit and not
+    a dimensionless unit.'''
+    t1 = Table([["A"]], names="B")
+    out = StringIO()
+    ascii.write(t1, out, format="ipac")
+    t2 = ascii.read(out.getvalue(), format="ipac", guess=False)
+    assert t2["B"].unit == None
