@@ -13,7 +13,9 @@ from __future__ import absolute_import, division, print_function
 import re
 import os
 import sys
-
+import copy
+import time
+import warnings
 
 from . import core
 from . import basic
@@ -31,9 +33,12 @@ from . import fixedwidth
 from ...table import Table
 from ...utils.data import get_readable_fileobj
 from ...extern import six
+from ...utils.exceptions import AstropyWarning
+
+_read_trace = []
 
 try:
-    import yaml
+    import yaml  # pylint: disable=W0611
     HAS_YAML = True
 except ImportError:
     HAS_YAML = False
@@ -122,11 +127,14 @@ def get_reader(Reader=None, Inputter=None, Outputter=None, **kwargs):
     quotechar : str
         One-character string to quote fields containing special characters
     header_start : int
-        Line index for the header line not counting comment lines
+        Line index for the header line not counting comment or blank lines.
+        A line with only whitespace is considered blank.
     data_start : int
-        Line index for the start of data not counting comment lines
+        Line index for the start of data not counting comment or blank lines.
+        A line with only whitespace is considered blank.
     data_end : int
-        Line index for the end of data (can be negative to count from end)
+        Line index for the end of data not counting comment or blank lines.
+        This value can be negative to count from the end.
     converters : dict
         Dictionary of converters
     data_Splitter : `~astropy.io.ascii.BaseSplitter`
@@ -146,6 +154,10 @@ def get_reader(Reader=None, Inputter=None, Outputter=None, **kwargs):
     fill_exclude_names : list
         List of names to exclude from fill_values (applied after ``fill_include_names``)
 
+    Returns
+    -------
+    reader : `~astropy.io.ascii.BaseReader` subclass
+        ASCII format reader instance
     """
     # This function is a light wrapper around core._get_reader to provide a public interface
     # with a default Reader.
@@ -176,9 +188,9 @@ def read(table, guess=None, **kwargs):
 
     Parameters
     ----------
-    table : str, file-like, list
-        Input table as a file name, file-like object, list of strings, or
-        single newline-separated string.
+    table : str, file-like, list, pathlib.Path object
+        Input table as a file name, file-like object, list of strings,
+        single newline-separated string or pathlib.Path object .
     guess : bool
         Try to guess the table format (default= ``True``)
     format : str, `~astropy.io.ascii.BaseReader`
@@ -194,11 +206,14 @@ def read(table, guess=None, **kwargs):
     quotechar : str
         One-character string to quote fields containing special characters
     header_start : int
-        Line index for the header line not counting comment lines
+        Line index for the header line not counting comment or blank lines.
+        A line with only whitespace is considered blank.
     data_start : int
-        Line index for the start of data not counting comment lines
+        Line index for the start of data not counting comment or blank lines.
+        A line with only whitespace is considered blank.
     data_end : int
-        Line index for the end of data (can be negative to count from end)
+        Line index for the end of data not counting comment or blank lines.
+        This value can be negative to count from the end.
     converters : dict
         Dictionary of converters
     data_Splitter : `~astropy.io.ascii.BaseSplitter`
@@ -222,7 +237,13 @@ def read(table, guess=None, **kwargs):
         (default= ``True``)
     Reader : `~astropy.io.ascii.BaseReader`
         Reader class (DEPRECATED) (default= :class:`Basic`).
+
+    Returns
+    -------
+    dat : `~astropy.table.Table`
+        Output table
     """
+    del _read_trace[:]
 
     if 'fill_values' not in kwargs:
         kwargs['fill_values'] = [('', '0')]
@@ -255,78 +276,132 @@ def read(table, guess=None, **kwargs):
         # file, thus prompting removal of the HTML reader from the default
         # guess list.
         new_kwargs['guess_html'] = _probably_html(table)
+
+        # If `table` is a filename or readable file object then read in the
+        # file now.  This prevents problems in Python 3 with the file object
+        # getting closed or left at the file end.  See #3132, #3013, #3109,
+        # #2001.  If a `readme` arg was passed that implies CDS format, in
+        # which case the original `table` as the data filename must be left
+        # intact.
+        if 'readme' not in new_kwargs:
+            try:
+                with get_readable_fileobj(table) as fileobj:
+                    table = fileobj.read()
+            except ValueError:  # unreadable or invalid binary file
+                raise
+            except:
+                pass
+            else:
+                # Ensure that `table` has at least one \r or \n in it
+                # so that the core.BaseInputter test of
+                # ('\n' not in table and '\r' not in table)
+                # will fail and so `table` cannot be interpreted there
+                # as a filename.  See #4160.
+                if not re.search(r'[\r\n]', table):
+                    table = table + os.linesep
+
+                # If the table got successfully read then look at the content
+                # to see if is probably HTML, but only if it wasn't already
+                # identified as HTML based on the filename.
+                if not new_kwargs['guess_html']:
+                    new_kwargs['guess_html'] = _probably_html(table)
+
+        # Get the table from guess in ``dat``.  If ``dat`` comes back as None
+        # then there was just one set of kwargs in the guess list so fall
+        # through below to the non-guess way so that any problems result in a
+        # more useful traceback.
         dat = _guess(table, new_kwargs, format, fast_reader_param)
-    else:
+        if dat is None:
+            guess = False
+
+    if not guess:
         reader = get_reader(**new_kwargs)
-        # Try the fast reader first if applicable
+        # Try the fast reader version of `format` first if applicable.  Note that
+        # if user specified a fast format (e.g. format='fast_basic') this test
+        # will fail and the else-clause below will be used.
         if fast_reader_param and format is not None and 'fast_{0}'.format(format) \
                                                         in core.FAST_CLASSES:
-            new_kwargs['Reader'] = core.FAST_CLASSES['fast_{0}'.format(format)]
-            fast_reader = get_reader(**new_kwargs)
+            fast_kwargs = copy.copy(new_kwargs)
+            fast_kwargs['Reader'] = core.FAST_CLASSES['fast_{0}'.format(format)]
+            fast_reader = get_reader(**fast_kwargs)
             try:
-                return fast_reader.read(table)
+                dat = fast_reader.read(table)
+                _read_trace.append({'kwargs': fast_kwargs,
+                                    'status': 'Success with fast reader (no guessing)'})
             except (core.ParameterError, cparser.CParserError) as e:
                 # special testing value to avoid falling back on the slow reader
                 if fast_reader_param == 'force':
                     raise e
                 # If the fast reader doesn't work, try the slow version
                 dat = reader.read(table)
+                _read_trace.append({'kwargs': new_kwargs,
+                                    'status': 'Success with slow reader after failing'
+                                             ' with fast (no guessing)'})
         else:
             dat = reader.read(table)
+            _read_trace.append({'kwargs': new_kwargs,
+                                'status': 'Success with specified Reader class '
+                                          '(no guessing)'})
 
     return dat
 
 
 def _guess(table, read_kwargs, format, fast_reader):
-    """Try to read the table using various sets of keyword args. First try the
-    original args supplied in the read() call. Then try the standard guess
-    keyword args. For each key/val pair specified explicitly in the read()
-    call make sure that if there is a corresponding definition in the guess
-    then it must have the same val.  If not then skip this guess."""
+    """
+    Try to read the table using various sets of keyword args.  Start with the
+    standard guess list and filter to make it unique and consistent with
+    user-supplied read keyword args.  Finally, if none of those work then
+    try the original user-supplied keyword args.
 
-    # If `table` is a readable file object then read in the file now.  This
-    # prevents problems in Python 3 with the file object getting closed or
-    # left at the file end.  See #3132, #3013, #3109, #2001.  If a `readme`
-    # arg was passed that implies CDS format, in which case the original
-    # `table` as the data filename must be left intact.
-    if 'readme' not in read_kwargs:
-        try:
-            with get_readable_fileobj(table) as fileobj:
-                table = fileobj.read()
-        except ValueError:  # unreadable or invalid binary file
-            raise
-        except:
-            pass
-        else:
-            # Ensure that `table` has at least one \r or \n in it
-            # so that the core.BaseInputter test of
-            # ('\n' not in table and '\r' not in table)
-            # will fail and so `table` cannot be interpreted there
-            # as a filename.  See #4160.
-            if not re.search(r'[\r\n]', table):
-                table = table + os.linesep
+    Parameters
+    ----------
+    table : str, file-like, list
+        Input table as a file name, file-like object, list of strings, or
+        single newline-separated string.
+    read_kwargs : dict
+        Keyword arguments from user to be supplied to reader
+    format : str
+        Table format
+    fast_reader : bool
+        Whether to use the C engine, can also be a dict with options which
+        default to ``False`` (default= ``True``)
 
-            # If the table got successfully read then look at the content
-            # to see if is probably HTML, but only if it wasn't already
-            # identified as HTML based on the filename.
-            if not read_kwargs['guess_html']:
-                read_kwargs['guess_html'] = _probably_html(table)
+    Returns
+    -------
+    dat : `~astropy.table.Table` or None
+        Output table or None if only one guess format was available
+    """
 
     # Keep a trace of all failed guesses kwarg
     failed_kwargs = []
+
+    # Get an ordered list of read() keyword arg dicts that will be cycled
+    # through in order to guess the format.
     full_list_guess = _get_guess_kwargs_list(read_kwargs)
 
+    # If a fast version of the reader is available, try that before the slow version
     if fast_reader and format is not None and 'fast_{0}'.format(format) in \
                                                          core.FAST_CLASSES:
-        # If a fast version of the reader is available, try that before the slow version
         fast_kwargs = read_kwargs.copy()
         fast_kwargs['Reader'] = core.FAST_CLASSES['fast_{0}'.format(format)]
         full_list_guess = [fast_kwargs] + full_list_guess
     else:
         fast_kwargs = None
 
-    # First try guessing
+    # Filter the full guess list so that each entry is consistent with user kwarg inputs.
+    # This also removes any duplicates from the list.
+    filtered_guess_kwargs = []
+    fast_reader = read_kwargs.get('fast_reader')
+
     for guess_kwargs in full_list_guess:
+        # If user specified slow reader then skip all fast readers
+        if fast_reader is False and guess_kwargs['Reader'] in core.FAST_CLASSES.values():
+            continue
+
+        # If user required a fast reader with 'force' then skip all non-fast readers
+        if fast_reader == 'force' and guess_kwargs['Reader'] not in core.FAST_CLASSES.values():
+            continue
+
         guess_kwargs_ok = True  # guess_kwargs are consistent with user_kwargs?
         for key, val in read_kwargs.items():
             # Do guess_kwargs.update(read_kwargs) except that if guess_args has
@@ -342,6 +417,30 @@ def _guess(table, read_kwargs, format, fast_reader):
             # user supplies delimiter="|" but the guess wants to try delimiter=" ",
             # so skip the guess entirely.
             continue
+
+        # Add the guess_kwargs to filtered list only if it is not already there.
+        if guess_kwargs not in filtered_guess_kwargs:
+            filtered_guess_kwargs.append(guess_kwargs)
+
+    # If there are not at least two formats to guess then return no table
+    # (None) to indicate that guessing did not occur.  In that case the
+    # non-guess read() will occur and any problems will result in a more useful
+    # traceback.
+    if len(filtered_guess_kwargs) <= 1:
+        return None
+
+    # Define whitelist of exceptions that are expected from readers when
+    # processing invalid inputs.  Note that IOError must fall through here
+    # so one cannot simply catch any exception.
+    guess_exception_classes = (core.InconsistentTableError, ValueError, TypeError,
+                               AttributeError, core.OptionalTableImportError,
+                               core.ParameterError, cparser.CParserError)
+
+    # Now cycle through each possible reader and associated keyword arguments.
+    # Try to read the table using those args, and if an exception occurs then
+    # keep track of the failed guess and move on.
+    for guess_kwargs in filtered_guess_kwargs:
+        t0 = time.time()
         try:
             # If guessing will try all Readers then use strict req'ts on column names
             if 'Reader' not in read_kwargs:
@@ -349,18 +448,31 @@ def _guess(table, read_kwargs, format, fast_reader):
 
             reader = get_reader(**guess_kwargs)
             reader.guessing = True
-            return reader.read(table)
+            dat = reader.read(table)
+            _read_trace.append({'kwargs': guess_kwargs, 'status': 'Success (guessing)',
+                                'dt': '{0:.3f} ms'.format((time.time() - t0) * 1000)})
+            return dat
 
-        except (core.InconsistentTableError, ValueError, TypeError, AttributeError,
-                core.OptionalTableImportError, core.ParameterError, cparser.CParserError):
+        except guess_exception_classes as err:
+            _read_trace.append({'kwargs': guess_kwargs,
+                                'status': '{0}: {1}'.format(err.__class__.__name__,
+                                                            str(err)),
+                                'dt': '{0:.3f} ms'.format((time.time() - t0) * 1000)})
             failed_kwargs.append(guess_kwargs)
     else:
-        # failed all guesses, try the original read_kwargs without column requirements
+        # Failed all guesses, try the original read_kwargs without column requirements
         try:
             reader = get_reader(**read_kwargs)
-            return reader.read(table)
-        except (core.InconsistentTableError, ValueError, ImportError,
-                core.OptionalTableImportError, core.ParameterError, cparser.CParserError):
+            dat = reader.read(table)
+            _read_trace.append({'kwargs': read_kwargs,
+                                'status': 'Success with original kwargs without strict_names '
+                                          '(guessing)'})
+            return dat
+
+        except guess_exception_classes as err:
+            _read_trace.append({'kwargs': guess_kwargs,
+                                'status': '{0}: {1}'.format(err.__class__.__name__,
+                                                            str(err))})
             failed_kwargs.append(read_kwargs)
             lines = ['\nERROR: Unable to guess table format with the guesses listed below:']
             for kwargs in failed_kwargs:
@@ -384,6 +496,31 @@ def _guess(table, read_kwargs, format, fast_reader):
             raise core.InconsistentTableError('\n'.join(lines))
 
 def _get_guess_kwargs_list(read_kwargs):
+    """
+    Get the full list of reader keyword argument dicts that are the basis
+    for the format guessing process.  The returned full list will then be:
+
+    - Filtered to be consistent with user-supplied kwargs
+    - Cleaned to have only unique entries
+    - Used one by one to try reading the input table
+
+    Note that the order of the guess list has been tuned over years of usage.
+    Maintainers need to be very careful about any adjustments as the
+    reasoning may not be immediately evident in all cases.
+
+    This list can (and usually does) include duplicates.  This is a result
+    of the order tuning, but these duplicates get removed later.
+
+    Parameters
+    ----------
+    read_kwargs : dict
+       User-supplied read keyword args
+
+    Returns
+    -------
+    guess_kwargs_list : list
+        List of read format keyword arg dicts
+    """
     guess_kwargs_list = []
 
     # If the table is probably HTML based on some heuristics then start with the
@@ -401,6 +538,7 @@ def _get_guess_kwargs_list(read_kwargs):
     # FixedWidthTwoLine would also be read by Basic, so it needs to come first.
     if len(read_kwargs) > 0:
         for reader in [fixedwidth.FixedWidthTwoLine,
+                       fastbasic.FastBasic,
                        basic.Basic]:
             first_kwargs = read_kwargs.copy()
             first_kwargs.update(dict(Reader=reader))
@@ -421,12 +559,16 @@ def _get_guess_kwargs_list(read_kwargs):
                               dict(Reader=latex.AASTex)
                               ])
 
-    for Reader in (basic.CommentedHeader, fastbasic.FastBasic, basic.Basic,
+    # Cycle through the basic-style readers using all combinations of delimiter
+    # and quotechar.
+    for Reader in (fastbasic.FastCommentedHeader, basic.CommentedHeader,
+                   fastbasic.FastBasic, basic.Basic,
                    fastbasic.FastNoHeader, basic.NoHeader):
         for delimiter in ("|", ",", " ", "\s"):
             for quotechar in ('"', "'"):
                 guess_kwargs_list.append(dict(
                     Reader=Reader, delimiter=delimiter, quotechar=quotechar))
+
     return guess_kwargs_list
 
 extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
@@ -461,6 +603,10 @@ def get_writer(Writer=None, fast_writer=True, **kwargs):
     fast_writer : bool
         Whether to use the fast Cython writer (default= ``True``)
 
+    Returns
+    -------
+    writer : `~astropy.io.ascii.BaseReader` subclass
+        ASCII format writer instance
     """
     if Writer is None:
         Writer = basic.Basic
@@ -522,6 +668,16 @@ def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwa
 
     table = Table(table, names=kwargs.get('names'))
 
+    table0 = table[:0].copy()
+    core._apply_include_exclude_names(table0, kwargs.get('names'),
+                    kwargs.get('include_names'), kwargs.get('exclude_names'))
+    diff_format_with_names = set(kwargs.get('formats', [])) - set(table0.colnames)
+
+    if diff_format_with_names:
+        warnings.warn(
+            'The keys {} specified in the formats argument does not match a column name.'
+            .format(diff_format_with_names), AstropyWarning)
+
     if table.has_mixin_columns:
         fast_writer = False
 
@@ -543,3 +699,20 @@ def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwa
     else:
         output.write(outstr)
         output.write(os.linesep)
+
+def get_read_trace():
+    """
+    Return a traceback of the attempted read formats for the last call to
+    `~astropy.io.ascii.read` where guessing was enabled.  This is primarily for
+    debugging.
+
+    The return value is a list of dicts, where each dict includes the keyword
+    args ``kwargs`` used in the read call and the returned ``status``.
+
+    Returns
+    -------
+    trace : list of dicts
+       Ordered list of format guesses and status
+    """
+
+    return copy.deepcopy(_read_trace)
