@@ -5,13 +5,13 @@ import warnings
 
 import numpy as np
 
-from .base import DELAYED, _ValidHDU, ExtensionHDU
+from .base import DELAYED, _ValidHDU, ExtensionHDU, BITPIX2DTYPE, DTYPE2BITPIX
 from ..header import Header
 from ..util import _is_pseudo_unsigned, _unsigned_zero, _is_int
 from ..verify import VerifyWarning
 
 from ....extern.six import string_types
-from ....utils import isiterable, lazyproperty
+from ....utils import isiterable, lazyproperty, classproperty, deprecated
 
 
 class _ImageBaseHDU(_ValidHDU):
@@ -26,14 +26,6 @@ class _ImageBaseHDU(_ValidHDU):
         image data
     """
 
-    # mappings between FITS and numpy typecodes
-    # TODO: Maybe make these module-level constants instead...
-    NumCode = {8: 'uint8', 16: 'int16', 32: 'int32', 64: 'int64',
-               -32: 'float32', -64: 'float64'}
-    ImgCode = {'uint8': 8, 'int16': 16, 'uint16': 16, 'int32': 32,
-               'uint32': 32, 'int64': 64, 'uint64': 64, 'float32': -32,
-               'float64': -64}
-
     standard_keyword_comments = {
         'SIMPLE': 'conforms to FITS standard',
         'XTENSION': 'Image extension',
@@ -45,7 +37,7 @@ class _ImageBaseHDU(_ValidHDU):
     }
 
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
-                 uint=False, scale_back=False, ignore_blank=False, **kwargs):
+                 uint=True, scale_back=False, ignore_blank=False, **kwargs):
 
         from .groups import GroupsHDU
 
@@ -107,27 +99,35 @@ class _ImageBaseHDU(_ValidHDU):
         self._uint = uint
         self._scale_back = scale_back
 
-        if do_not_scale_image_data:
-            self._bzero = 0
-            self._bscale = 1
-        else:
-            self._bzero = self._header.get('BZERO', 0)
-            self._bscale = self._header.get('BSCALE', 1)
+        # Keep track of whether BZERO/BSCALE were set from the header so that
+        # values for self._orig_bzero and self._orig_bscale can be set
+        # properly, if necessary, once the data has been set.
+        bzero_in_header = 'BZERO' in self._header
+        bscale_in_header = 'BSCALE' in self._header
+        self._bzero = self._header.get('BZERO', 0)
+        self._bscale = self._header.get('BSCALE', 1)
 
         # Save off other important values from the header needed to interpret
         # the image data
         self._axes = [self._header.get('NAXIS' + str(axis + 1), 0)
                       for axis in range(self._header.get('NAXIS', 0))]
-        self._bitpix = self._header.get('BITPIX', 8)
+
+        # Not supplying a default for BITPIX makes sense because BITPIX
+        # is either in the header or should be determined from the dtype of
+        # the data (which occurs when the data is set).
+        self._bitpix = self._header.get('BITPIX')
         self._gcount = self._header.get('GCOUNT', 1)
         self._pcount = self._header.get('PCOUNT', 0)
         self._blank = None if ignore_blank else self._header.get('BLANK')
         self._verify_blank()
 
         self._orig_bitpix = self._bitpix
+        self._orig_blank = self._header.get('BLANK')
+
+        # These get set again below, but need to be set to sensible defaults
+        # here.
         self._orig_bzero = self._bzero
         self._orig_bscale = self._bscale
-        self._orig_blank = self._header.get('BLANK')
 
         # Set the name attribute if it was provided (if this is an ImageHDU
         # this will result in setting the EXTNAME keyword of the header as
@@ -147,9 +147,31 @@ class _ImageBaseHDU(_ValidHDU):
                 self._data_needs_rescale = True
             return
         else:
+            # Setting data will set set _bitpix, _bzero, and _bscale to the
+            # appropriate BITPIX for the data, and always sets _bzero=0 and
+            # _bscale=1.
             self.data = data
             self.update_header()
 
+            # Check again for BITPIX/BSCALE/BZERO in case they changed when the
+            # data was assigned. This can happen, for example, if the input
+            # data is an unsigned int numpy array.
+            self._bitpix = self._header.get('BITPIX')
+
+            # Do not provide default values for BZERO and BSCALE here because
+            # the keywords will have been deleted in the header if appropriate
+            # after scaling. We do not want to put them back in if they
+            # should not be there.
+            self._bzero = self._header.get('BZERO')
+            self._bscale = self._header.get('BSCALE')
+
+        # Handle case where there was no BZERO/BSCALE in the initial header
+        # but there should be a BSCALE/BZERO now that the data has been set.
+        if not bzero_in_header:
+            self._orig_bzero = self._bzero
+        if bscale_in_header:
+            self._orig_bscale = self._bscale
+
     @classmethod
     def match_header(cls, header):
         """
@@ -224,13 +246,15 @@ class _ImageBaseHDU(_ValidHDU):
 
     @data.setter
     def data(self, data):
-        if 'data' in self.__dict__:
+        if 'data' in self.__dict__ and self.__dict__['data'] is not None:
             if self.__dict__['data'] is data:
                 return
             else:
                 self._data_replaced = True
+            was_unsigned = _is_pseudo_unsigned(self.__dict__['data'].dtype)
         else:
             self._data_replaced = True
+            was_unsigned = False
 
         if data is not None and not isinstance(data, np.ndarray):
             # Try to coerce the data into a numpy array--this will work, on
@@ -245,10 +269,12 @@ class _ImageBaseHDU(_ValidHDU):
         self._modified = True
 
         if isinstance(data, np.ndarray):
-            self._bitpix = _ImageBaseHDU.ImgCode[data.dtype.name]
-            self._orig_bitpix = self._bitpix
-            self._orig_bscale = 1
-            self._orig_bzero = 0
+            # Set new values of bitpix, bzero, and bscale now, but wait to
+            # revise original values until header is updated.
+            self._bitpix = DTYPE2BITPIX[data.dtype.name]
+            self._bscale = 1
+            self._bzero = 0
+            self._blank = None
             self._axes = list(data.shape)
             self._axes.reverse()
         elif self.data is None:
@@ -256,7 +282,26 @@ class _ImageBaseHDU(_ValidHDU):
         else:
             raise ValueError('not a valid data array')
 
+        # Update the header, including adding BZERO/BSCALE if new data is
+        # unsigned. Does not change the values of self._bitpix,
+        # self._orig_bitpix, etc.
         self.update_header()
+        if (data is not None and was_unsigned):
+            self._update_header_scale_info(data.dtype)
+
+        # Keep _orig_bitpix as it was until header update is done, then
+        # set it, to allow easier handling of the case of unsigned
+        # integer data being converted to something else. Setting these here
+        # is needed only for the case do_not_scale_image_data=True when
+        # setting the data to unsigned int.
+
+        # If necessary during initialization, i.e. if BSCALE and BZERO were
+        # not in the header but the data was unsigned, the attributes below
+        # will be update in __init__.
+        self._orig_bitpix = self._bitpix
+        self._orig_bscale = self._bscale
+        self._orig_bzero = self._bzero
+
 
         # returning the data signals to lazyproperty that we've already handled
         # setting self.__dict__['data']
@@ -324,43 +369,63 @@ class _ImageBaseHDU(_ValidHDU):
         if 'BLANK' in self._header:
             self._blank = self._header['BLANK']
 
+        # Add BSCALE/BZERO to header if data is unsigned int.
         self._update_uint_scale_keywords()
 
         self._modified = False
 
     def _update_header_scale_info(self, dtype=None):
-        if (not self._do_not_scale_image_data and
-                not (self._orig_bzero == 0 and self._orig_bscale == 1)):
+        """
+        Delete BSCALE/BZERO from header if necessary.
+        """
 
-            if dtype is None:
-                dtype = self._dtype_for_bitpix()
+        # Note that _dtype_for_bitpix determines the dtype based on the
+        # "original" values of bitpix, bscale, and bzero, stored in
+        # self._orig_bitpix, etc. It contains the logic for determining which
+        # special cases of BZERO/BSCALE, if any, are auto-detected as following
+        # the FITS unsigned int convention.
 
-            if (dtype is not None and dtype.kind == 'u' and
-                    (self._scale_back or self._scale_back is None)):
-                # Data is pseudo-unsigned integers, and the scale_back option
-                # was not explicitly set to False, so preserve all the scale
-                # factors
-                return
+        # Added original_was_unsigned with the intent of facilitating the
+        # special case of do_not_scale_image_data=True and uint=True
+        # eventually.
+        if self._dtype_for_bitpix() is not None:
+            original_was_unsigned = self._dtype_for_bitpix().kind == 'u'
+        else:
+            original_was_unsigned = False
 
-            for keyword in ['BSCALE', 'BZERO']:
-                try:
-                    del self._header[keyword]
-                    # Since _update_header_scale_info can, currently, be called
-                    # *after* _prewriteto(), replace these with blank cards so
-                    # the header size doesn't change
-                    self._header.append()
-                except KeyError:
-                    pass
+        if (self._do_not_scale_image_data or
+                (self._orig_bzero == 0 and self._orig_bscale == 1)):
+            return
 
-            if dtype is None:
-                dtype = self._dtype_for_bitpix()
-            if dtype is not None:
-                self._header['BITPIX'] = _ImageBaseHDU.ImgCode[dtype.name]
+        if dtype is None:
+            dtype = self._dtype_for_bitpix()
 
-            self._bzero = 0
-            self._bscale = 1
-            self._bitpix = self._header['BITPIX']
-            self._blank = self._header.pop('BLANK', None)
+        if (dtype is not None and dtype.kind == 'u' and
+                (self._scale_back or self._scale_back is None)):
+            # Data is pseudo-unsigned integers, and the scale_back option
+            # was not explicitly set to False, so preserve all the scale
+            # factors
+            return
+
+        for keyword in ['BSCALE', 'BZERO']:
+            try:
+                del self._header[keyword]
+                # Since _update_header_scale_info can, currently, be called
+                # *after* _prewriteto(), replace these with blank cards so
+                # the header size doesn't change
+                self._header.append()
+            except KeyError:
+                pass
+
+        if dtype is None:
+            dtype = self._dtype_for_bitpix()
+        if dtype is not None:
+            self._header['BITPIX'] = DTYPE2BITPIX[dtype.name]
+
+        self._bzero = 0
+        self._bscale = 1
+        self._bitpix = self._header['BITPIX']
+        self._blank = self._header.pop('BLANK', None)
 
     def scale(self, type=None, option='old', bscale=1, bzero=0):
         """
@@ -415,7 +480,7 @@ class _ImageBaseHDU(_ValidHDU):
 
         # Determine the destination (numpy) data type
         if type is None:
-            type = self.NumCode[self._bitpix]
+            type = BITPIX2DTYPE[self._bitpix]
         _type = getattr(np, type)
 
         # Determine how to scale the data
@@ -477,7 +542,7 @@ class _ImageBaseHDU(_ValidHDU):
             self.data = np.array(np.around(self.data), dtype=_type)
 
         # Update the BITPIX Card to match the data
-        self._bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
+        self._bitpix = DTYPE2BITPIX[self.data.dtype.name]
         self._bzero = self._header.get('BZERO', 0)
         self._bscale = self._header.get('BSCALE', 1)
         self._blank = blank
@@ -527,7 +592,7 @@ class _ImageBaseHDU(_ValidHDU):
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._scale_back:
-            self._scale_internal(self.NumCode[self._orig_bitpix],
+            self._scale_internal(BITPIX2DTYPE[self._orig_bitpix],
                                  blank=self._orig_blank)
 
         self.update_header()
@@ -626,7 +691,7 @@ class _ImageBaseHDU(_ValidHDU):
         supports alternate offset/shape for Section support.
         """
 
-        code = _ImageBaseHDU.NumCode[self._orig_bitpix]
+        code = BITPIX2DTYPE[self._orig_bitpix]
 
         raw_data = self._get_raw_data(shape, code, offset)
         raw_data.dtype = raw_data.dtype.newbyteorder('>')
@@ -692,7 +757,6 @@ class _ImageBaseHDU(_ValidHDU):
 
         return data
 
-    # TODO: Move the GroupsHDU-specific summary code to GroupsHDU itself
     def _summary(self):
         """
         Summarize the HDU: name, dimensions, and formats.
@@ -711,10 +775,15 @@ class _ImageBaseHDU(_ValidHDU):
             if self.shape and all(self.shape):
                 # Only show the format if all the dimensions are non-zero
                 # if data is not touched yet, use header info.
-                format = self.NumCode[self._bitpix]
+                format = BITPIX2DTYPE[self._bitpix]
             else:
                 format = ''
 
+            if (format and not self._do_not_scale_image_data and
+                    (self._orig_bscale != 1 or self._orig_bzero != 0)):
+                new_dtype = self._dtype_for_bitpix()
+                format += ' (rescales to {0})'.format(new_dtype.name)
+
         # Display shape in FITS-order
         shape = tuple(reversed(self.shape))
 
@@ -762,6 +831,18 @@ class _ImageBaseHDU(_ValidHDU):
             return super(_ImageBaseHDU, self)._calculate_datasum(
                 blocking=blocking)
 
+    @classproperty
+    @deprecated('1.1.0', alternative='the module level constant BITPIX2DTYPE',
+                obj_type='class attribute')
+    def NumCode(cls):
+        return BITPIX2DTYPE
+
+    @classproperty
+    @deprecated('1.1.0', alternative='the module level constant DTYPE2BITPIX',
+                obj_type='class attribute')
+    def ImgCode(cls):
+        return DTYPE2BITPIX
+
 
 class Section(object):
     """
@@ -866,7 +947,7 @@ class PrimaryHDU(_ImageBaseHDU):
 
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
                  ignore_blank=False,
-                 uint=False, scale_back=None):
+                 uint=True, scale_back=None):
         """
         Construct a primary HDU.
 
@@ -881,25 +962,28 @@ class PrimaryHDU(_ImageBaseHDU):
 
         do_not_scale_image_data : bool, optional
             If `True`, image data is not scaled using BSCALE/BZERO values
-            when read.
+            when read. (default: False)
 
         ignore_blank : bool, optional
             If `True`, the BLANK header keyword will be ignored if present.
             Otherwise, pixels equal to this value will be replaced with
-            NaNs.
+            NaNs. (default: False)
 
         uint : bool, optional
             Interpret signed integer data where ``BZERO`` is the
             central value and ``BSCALE == 1`` as unsigned integer
             data.  For example, ``int16`` data with ``BZERO = 32768``
             and ``BSCALE = 1`` would be treated as ``uint16`` data.
+            (default: True)
 
         scale_back : bool, optional
             If `True`, when saving changes to a file that contained scaled
             image data, restore the data to the original type and reapply the
             original BSCALE/BZERO values.  This could lead to loss of accuracy
             if scaling back to integer values after performing floating point
-            operations on the data.
+            operations on the data.  Pseudo-unsigned integers are automatically
+            rescaled unless scale_back is explicitly set to `False`.
+            (default: None)
         """
 
         super(PrimaryHDU, self).__init__(
@@ -955,7 +1039,7 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
     _extension = 'IMAGE'
 
     def __init__(self, data=None, header=None, name=None,
-                 do_not_scale_image_data=False, uint=False, scale_back=None):
+                 do_not_scale_image_data=False, uint=True, scale_back=None):
         """
         Construct an image HDU.
 
@@ -974,20 +1058,23 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
 
         do_not_scale_image_data : bool, optional
             If `True`, image data is not scaled using BSCALE/BZERO values
-            when read.
+            when read. (default: False)
 
         uint : bool, optional
             Interpret signed integer data where ``BZERO`` is the
             central value and ``BSCALE == 1`` as unsigned integer
             data.  For example, ``int16`` data with ``BZERO = 32768``
             and ``BSCALE = 1`` would be treated as ``uint16`` data.
+            (default: True)
 
         scale_back : bool, optional
             If `True`, when saving changes to a file that contained scaled
             image data, restore the data to the original type and reapply the
             original BSCALE/BZERO values.  This could lead to loss of accuracy
             if scaling back to integer values after performing floating point
-            operations on the data.
+            operations on the data.  Pseudo-unsigned integers are automatically
+            rescaled unless scale_back is explicitly set to `False`.
+            (default: None)
         """
 
         # This __init__ currently does nothing differently from the base class,
