@@ -4,21 +4,17 @@ from __future__ import division
 
 import collections
 import copy
-import inspect
 import itertools
 import re
-import sys
 import warnings
 
-from collections import defaultdict
-
-from .card import Card, CardList, _pad, BLANK_CARD, KEYWORD_LENGTH
+from .card import Card, CardList, _pad, KEYWORD_LENGTH
 from .file import _File
-from .util import (encode_ascii, decode_ascii, fileobj_mode, fileobj_closed,
+from .util import (encode_ascii, decode_ascii, fileobj_closed,
                    fileobj_is_binary)
 
 from ...extern import six
-from ...extern.six import PY3, string_types, itervalues, iteritems, next
+from ...extern.six import string_types, itervalues, iteritems, next
 from ...extern.six.moves import zip, range, zip_longest
 from ...utils import deprecated, isiterable
 from ...utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
@@ -27,7 +23,7 @@ from ...utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
 BLOCK_SIZE = 2880  # the FITS block size
 
 # This regular expression can match a *valid* END card which just consists of
-# the the string 'END' followed by all spaces, or an *invalid* end card which
+# the string 'END' followed by all spaces, or an *invalid* end card which
 # consists of END, followed by any character that is *not* a valid character
 # for a valid FITS keyword (that is, this is not a keyword like 'ENDER' which
 # starts with 'END' but is not 'END'), followed by any arbitrary bytes.  An
@@ -88,7 +84,8 @@ class Header(object):
         Parameters
         ----------
         cards : A list of `Card` objects, optional
-            The cards to initialize the header with.
+            The cards to initialize the header with. Also allowed are other
+            `Header` (or `dict`-like) objects.
 
         txtfile : file path, file object or file-like object, optional
             Input ASCII header parameters file **(Deprecated)**
@@ -109,6 +106,8 @@ class Header(object):
 
         if isinstance(cards, Header):
             cards = cards.cards
+        elif isinstance(cards, dict):
+            cards = six.iteritems(cards)
 
         for card in cards:
             self.append(card, end=True)
@@ -126,7 +125,7 @@ class Header(object):
         if keyword in self._keyword_indices or keyword in self._rvkc_indices:
             # For the most common case (single, standard form keyword lookup)
             # this will work and is an O(1) check.  If it fails that doesn't
-            # guarantee absense, just that we have to perform the full set of
+            # guarantee absence, just that we have to perform the full set of
             # checks in self._cardindex
             return True
         try:
@@ -141,7 +140,7 @@ class Header(object):
         elif self._haswildcard(key):
             return Header([copy.copy(self._cards[idx])
                            for idx in self._wildcardmatch(key)])
-        elif (isinstance(key, basestring) and
+        elif (isinstance(key, string_types) and
               key.upper() in Card._commentary_keywords):
             key = key.upper()
             # Special case for commentary cards
@@ -205,7 +204,7 @@ class Header(object):
             # the cards are deleted before updating _keyword_indices rather
             # than updating it once for each card that gets deleted]
             if isinstance(key, slice):
-                indices = xrange(*key.indices(len(self)))
+                indices = range(*key.indices(len(self)))
                 # If the slice step is backwards we want to reverse it, because
                 # it will be reversed in a few lines...
                 if key.step and key.step < 0:
@@ -215,7 +214,7 @@ class Header(object):
             for idx in reversed(indices):
                 del self[idx]
             return
-        elif isinstance(key, basestring):
+        elif isinstance(key, string_types):
             # delete ALL cards with the same keyword name
             key = Card.normalize_keyword(key)
             indices = self._keyword_indices
@@ -223,21 +222,11 @@ class Header(object):
                 indices = self._rvkc_indices
 
             if key not in indices:
-                if _is_astropy_internal():
-                    # All internal code is designed to assume that this will
-                    # raise a KeyError, so go ahead and do so
-                    raise KeyError("Keyword '%s' not found." % key)
-                # Warn everyone else.
-                # TODO: Remove this warning and make KeyError the default after
-                # a couple versions (by 3.3, say)
-                warnings.warn(
-                    'Deletion of non-existent keyword %r: '
-                    'In a future Astropy version Header.__delitem__ may be '
-                    'changed so that this raises a KeyError just like a dict '
-                    'would. Please update your code so that KeyErrors are '
-                    'caught and handled when deleting non-existent keywords.' %
-                    key, AstropyDeprecationWarning)
-                return
+                # if keyword is not present raise KeyError.
+                # To delete keyword without caring if they were present,
+                # Header.remove(Keyword) can be used with optional argument ignore_missing as True
+                raise KeyError("Keyword '%s' not found." % key)
+
             for idx in reversed(indices[key]):
                 # Have to copy the indices list since it will be modified below
                 del self[idx]
@@ -424,8 +413,8 @@ class Header(object):
 
         endcard : bool, optional
             If True (the default) the header must end with an END card in order
-            to be considered valid.  If an END card is not found an `IOError`
-            is raised.
+            to be considered valid.  If an END card is not found an
+            `IOError` is raised.
 
         padding : bool, optional
             If True (the default) the header will be required to be padded out
@@ -439,7 +428,7 @@ class Header(object):
         """
 
         close_file = False
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             # Open in text mode by default to support newline handling; if a
             # binary-mode file object is passed in, the user is on their own
             # with respect to newline handling
@@ -447,35 +436,58 @@ class Header(object):
             close_file = True
 
         try:
-            return cls._fromfile_internal(fileobj, sep, endcard, padding)
+            is_binary = fileobj_is_binary(fileobj)
+
+            def block_iter(nbytes):
+                while True:
+                    data = fileobj.read(nbytes)
+
+                    if data:
+                        yield data
+                    else:
+                        break
+
+            return cls._from_blocks(block_iter, is_binary, sep, endcard,
+                                    padding)[1]
         finally:
             if close_file:
                 fileobj.close()
 
     @classmethod
-    def _fromfile_internal(cls, fileobj, sep, endcard, padding):
+    def _from_blocks(cls, block_iter, is_binary, sep, endcard, padding):
         """
         The meat of `Header.fromfile`; in a separate method so that
         `Header.fromfile` itself is just responsible for wrapping file
-        handling.
+        handling.  Also used by `_BaseHDU.fromstring`.
+
+        ``block_iter`` should be a callable which, given a block size n
+        (typically 2880 bytes as used by the FITS standard) returns an iterator
+        of byte strings of that block size.
+
+        ``is_binary`` specifies whether the returned blocks are bytes or text
+
+        Returns both the entire header *string*, and the `Header` object
+        returned by Header.fromstring on that string.
         """
 
-        is_binary = fileobj_is_binary(fileobj)
         actual_block_size = _block_size(sep)
         clen = Card.length + len(sep)
 
+        blocks = block_iter(actual_block_size)
+
         # Read the first header block.
-        block = fileobj.read(actual_block_size)
+        try:
+            block = next(blocks)
+        except StopIteration:
+            raise EOFError()
+
         if not is_binary:
             # TODO: There needs to be error handling at *this* level for
             # non-ASCII characters; maybe at this stage decoding latin-1 might
             # be safer
             block = encode_ascii(block)
 
-        if not block:
-            raise EOFError()
-
-        blocks = []
+        read_blocks = []
         is_eof = False
         end_found = False
 
@@ -484,12 +496,16 @@ class Header(object):
             # find the END card
             end_found, block = cls._find_end_card(block, clen)
 
-            blocks.append(decode_ascii(block))
+            read_blocks.append(decode_ascii(block))
 
             if end_found:
                 break
 
-            block = fileobj.read(actual_block_size)
+            try:
+                block = next(blocks)
+            except StopIteration:
+                is_eof = True
+                break
 
             if not block:
                 is_eof = True
@@ -503,11 +519,11 @@ class Header(object):
             # rather than raising an exception
             raise IOError('Header missing END card.')
 
-        blocks = ''.join(blocks)
+        header_str = ''.join(read_blocks)
 
         # Strip any zero-padding (see ticket #106)
-        if blocks and blocks[-1] == '\0':
-            if is_eof and blocks.strip('\0') == '':
+        if header_str and header_str[-1] == '\0':
+            if is_eof and header_str.strip('\0') == '':
                 # TODO: Pass this warning to validation framework
                 warnings.warn(
                     'Unexpected extra padding at the end of the file.  This '
@@ -522,27 +538,27 @@ class Header(object):
                     'Header block contains null bytes instead of spaces for '
                     'padding, and is not FITS-compliant. Nulls may be '
                     'replaced with spaces upon writing.', AstropyUserWarning)
-                blocks.replace('\0', ' ')
+                header_str.replace('\0', ' ')
 
-        if padding and (len(blocks) % actual_block_size) != 0:
+        if padding and (len(header_str) % actual_block_size) != 0:
             # This error message ignores the length of the separator for
             # now, but maybe it shouldn't?
-            actual_len = len(blocks) - actual_block_size + BLOCK_SIZE
+            actual_len = len(header_str) - actual_block_size + BLOCK_SIZE
             # TODO: Pass this error to validation framework
             raise ValueError(
                 'Header size is not multiple of {0}: {1}'.format(BLOCK_SIZE,
                                                                  actual_len))
 
-        return cls.fromstring(blocks, sep=sep)
+        return header_str, cls.fromstring(header_str, sep=sep)
 
     @classmethod
     def _find_end_card(cls, block, card_len):
         """
-        Utitility method to search a header block for the END card and handle
+        Utility method to search a header block for the END card and handle
         invalid END cards.
 
         This method can also returned a modified copy of the input header block
-        in case an invalid end card needs to be sanitized
+        in case an invalid end card needs to be sanitized.
         """
 
         for mo in HEADER_END_RE.finditer(block):
@@ -592,7 +608,7 @@ class Header(object):
         ----------
         sep : str, optional
             The character or string with which to separate cards.  By default
-            there is no separator, but one could use `'\\n'`, for example, to
+            there is no separator, but one could use ``'\\n'``, for example, to
             separate each card with a new line
 
         endcard : bool, optional
@@ -642,7 +658,7 @@ class Header(object):
 
         sep : str, optional
             The character or string with which to separate cards.  By default
-            there is no separator, but one could use `'\\n'`, for example, to
+            there is no separator, but one could use ``'\\n'``, for example, to
             separate each card with a new line
 
         endcard : bool, optional
@@ -807,7 +823,7 @@ class Header(object):
             It should be noted that ``header.set(keyword, value)`` and
             ``header.set(keyword, value, comment)`` are equivalent to
             ``header[keyword] = value`` and
-            ``header[keyword] = (value, comment)`` respectfully.
+            ``header[keyword] = (value, comment)`` respectively.
 
             New keywords can also be inserted relative to existing keywords
             using, for example::
@@ -820,7 +836,7 @@ class Header(object):
 
             to insert after an existing keyword.
 
-            The the only advantage of using :meth:`Header.set` is that it
+            The only advantage of using :meth:`Header.set` is that it
             easily replaces the old usage of :meth:`Header.update` both
             conceptually and in terms of function signature.
 
@@ -884,7 +900,7 @@ class Header(object):
     def items(self):
         """Like :meth:`dict.items`."""
 
-        return list(self.iteritems())
+        return list(iteritems(self))
 
     def iteritems(self):
         """Like :meth:`dict.iteritems`."""
@@ -903,7 +919,7 @@ class Header(object):
     def itervalues(self):
         """Like :meth:`dict.itervalues`."""
 
-        for _, v in self.iteritems():
+        for _, v in iteritems(self):
             yield v
 
     def keys(self):
@@ -943,14 +959,14 @@ class Header(object):
         """Similar to :meth:`dict.popitem`."""
 
         try:
-            k, v = self.iteritems().next()
+            k, v = next(iteritems(self))
         except StopIteration:
             raise KeyError('Header is empty')
         del self[k]
         return k, v
 
     def setdefault(self, key, default=None):
-        """Similar to :meth:`dict.setitem`."""
+        """Similar to :meth:`dict.setdefault`."""
 
         try:
             return self[key]
@@ -1060,7 +1076,7 @@ class Header(object):
 
         savecomment : bool, optional
             When `True`, preserve the current comment for an existing
-            keyword.  The argument `savecomment` takes precedence over
+            keyword.  The argument ``savecomment`` takes precedence over
             ``comment`` if both specified.  If ``comment`` is not
             specified then the current comment will automatically be
             preserved.
@@ -1135,7 +1151,7 @@ class Header(object):
             if other is None:
                 pass
             elif hasattr(other, 'iteritems'):
-                for k, v in other.iteritems():
+                for k, v in iteritems(other):
                     update_from_dict(k, v)
             elif hasattr(other, 'keys'):
                 for k in other.keys():
@@ -1158,7 +1174,7 @@ class Header(object):
     def values(self):
         """Returns a list of the values of all cards in the header."""
 
-        return [v for _, v in self.iteritems()]
+        return [v for _, v in iteritems(self)]
 
     def append(self, card=None, useblanks=True, bottom=False, end=False):
         """
@@ -1195,7 +1211,7 @@ class Header(object):
 
         """
 
-        if isinstance(card, basestring):
+        if isinstance(card, string_types):
             card = Card(card)
         elif isinstance(card, tuple):
             card = Card(*card)
@@ -1206,7 +1222,7 @@ class Header(object):
                 'The value appended to a Header must be either a keyword or '
                 '(keyword, value, [comment]) tuple; got: %r' % card)
 
-        if not end and str(card) == BLANK_CARD:
+        if not end and card.is_blank:
             # Blank cards should always just be appended to the end
             end = True
 
@@ -1215,7 +1231,7 @@ class Header(object):
             idx = len(self._cards) - 1
         else:
             idx = len(self._cards) - 1
-            while idx >= 0 and str(self._cards[idx]) == BLANK_CARD:
+            while idx >= 0 and self._cards[idx].is_blank:
                 idx -= 1
 
             if not bottom and card.keyword not in Card._commentary_keywords:
@@ -1240,7 +1256,12 @@ class Header(object):
                 self._keyword_indices[keyword].sort()
 
             # Finally, if useblanks, delete a blank cards from the end
-            if useblanks:
+            if useblanks and self._countblanks():
+                # Don't do this unless there is at least one blanks at the end
+                # of the header; we need to convert the card to its string
+                # image to see how long it is.  In the vast majority of cases
+                # this will just be 80 (Card.length) but it may be longer for
+                # CONTINUE cards
                 self._useblanks(len(str(card)) // Card.length)
 
         self._modified = True
@@ -1303,7 +1324,7 @@ class Header(object):
         # We don't immediately modify the header, because first we need to sift
         # out any duplicates in the new header prior to adding them to the
         # existing header, but while *allowing* duplicates from the header
-        # being exteded from (see ticket #156)
+        # being extended from (see ticket #156)
         extend_cards = []
 
         for idx, card in enumerate(temp.cards):
@@ -1329,8 +1350,8 @@ class Header(object):
                 else:
                     extend_cards.append(card)
             else:
-                if unique or update and keyword in self:
-                    if str(card) == BLANK_CARD:
+                if (unique or update) and keyword in self:
+                    if card.is_blank:
                         extend_cards.append(card)
                         continue
 
@@ -1398,7 +1419,7 @@ class Header(object):
 
         norm_keyword = Card.normalize_keyword(keyword)
 
-        for idx in xrange(start, stop, step):
+        for idx in range(start, stop, step):
             if self._cards[idx].keyword.upper() == norm_keyword:
                 return idx
         else:
@@ -1412,7 +1433,7 @@ class Header(object):
         Parameters
         ----------
         key : int, str, or tuple
-            The index into the the list of header keywords before which the
+            The index into the list of header keywords before which the
             new keyword should be inserted, or the name of a keyword before
             which the new keyword should be inserted.  Can also accept a
             (keyword, index) tuple for inserting around duplicate keywords.
@@ -1453,7 +1474,7 @@ class Header(object):
             self.append(card, end=True)
             return
 
-        if isinstance(card, basestring):
+        if isinstance(card, string_types):
             card = Card(card)
         elif isinstance(card, tuple):
             card = Card(*card)
@@ -1492,25 +1513,39 @@ class Header(object):
             rvkc_indices.append(idx)
             rvkc_indices.sort()
 
-
         if useblanks:
             self._useblanks(len(str(card)) // Card.length)
 
         self._modified = True
 
-    def remove(self, keyword):
+    def remove(self, keyword, ignore_missing=False, remove_all=False):
         """
         Removes the first instance of the given keyword from the header similar
         to `list.remove` if the Header object is treated as a list of keywords.
 
         Parameters
         ----------
-        value : str
-            The keyword of which to remove the first instance in the header
+        keyword : str
+            The keyword of which to remove the first instance in the header.
+
+        ignore_missing : bool, optional
+            When True, ignores missing keywords.  Otherwise, if the keyword
+            is not present in the header a KeyError is raised.
+
+        remove_all : bool, optional
+            When True, all instances of keyword will be removed.
+            Otherwise only the first instance of the given keyword is removed.
 
         """
+        keyword = Card.normalize_keyword(keyword)
+        if keyword in self._keyword_indices:
+            del self[self._keyword_indices[keyword][0]]
+            if remove_all:
+                while keyword in self._keyword_indices:
+                    del self[self._keyword_indices[keyword][0]]
+        elif not ignore_missing:
+            raise KeyError("Keyword '%s' not found." % keyword)
 
-        del self[self.index(keyword)]
 
     def rename_keyword(self, oldkeyword, newkeyword, force=False):
         """
@@ -1526,8 +1561,8 @@ class Header(object):
 
         force : bool, optional
             When `True`, if the new keyword already exists in the header, force
-            the creation of a duplicate keyword.  Otherwise a `ValueError` is
-            raised.
+            the creation of a duplicate keyword. Otherwise a
+            `ValueError` is raised.
         """
 
         oldkeyword = Card.normalize_keyword(oldkeyword)
@@ -1629,7 +1664,7 @@ class Header(object):
             existing_card = self._cards[idx]
             existing_card.value = value
             if comment is not None:
-                # '' should be used to explictly blank a comment
+                # '' should be used to explicitly blank a comment
                 existing_card.comment = comment
             if existing_card._modified:
                 self._modified = True
@@ -1797,14 +1832,14 @@ class Header(object):
     def _countblanks(self):
         """Returns the number of blank cards at the end of the Header."""
 
-        for idx in xrange(1, len(self._cards)):
-            if str(self._cards[-idx]) != BLANK_CARD:
+        for idx in range(1, len(self._cards)):
+            if not self._cards[-idx].is_blank:
                 return idx - 1
         return 0
 
     def _useblanks(self, count):
         for _ in range(count):
-            if str(self._cards[-1]) == BLANK_CARD:
+            if self._cards[-1].is_blank:
                 del self[-1]
             else:
                 break
@@ -1812,7 +1847,7 @@ class Header(object):
     def _haswildcard(self, keyword):
         """Return `True` if the input keyword contains a wildcard pattern."""
 
-        return (isinstance(keyword, basestring) and
+        return (isinstance(keyword, string_types) and
                 (keyword.endswith('...') or '*' in keyword or '?' in keyword))
 
     def _wildcardmatch(self, pattern):
@@ -1820,12 +1855,12 @@ class Header(object):
         Returns a list of indices of the cards matching the given wildcard
         pattern.
 
-         * '*' matches 0 or more alphanumeric characters or _
-         * '?' matches a single alphanumeric character or _
+         * '*' matches 0 or more characters
+         * '?' matches a single character
          * '...' matches 0 or more of any non-whitespace character
         """
 
-        pattern = pattern.replace('*', r'\w*').replace('?', r'\w')
+        pattern = pattern.replace('*', r'.*').replace('?', r'.')
         pattern = pattern.replace('...', r'\S*') + '$'
         pattern_re = re.compile(pattern, re.I)
 
@@ -1932,7 +1967,7 @@ class Header(object):
         """
         Add a commentary card.
 
-        If `before` and `after` are `None`, add to the last occurrence
+        If ``before`` and ``after`` are `None`, add to the last occurrence
         of cards of the same name (except blank card).  If there is no
         card (or blank card), append at the end.
         """
@@ -1945,7 +1980,7 @@ class Header(object):
 
     # Some fixes for compatibility with the Python 3 dict interface, where
     # iteritems -> items, etc.
-    if PY3:  # pragma: py3
+    if six.PY3:
         keys = iterkeys
         values = itervalues
         items = iteritems
@@ -2143,12 +2178,12 @@ class _CardAccessor(object):
 
         if isinstance(item, slice) or self._header._haswildcard(item):
             if isinstance(item, slice):
-                indices = xrange(*item.indices(len(self)))
+                indices = range(*item.indices(len(self)))
             else:
                 indices = self._header._wildcardmatch(item)
-            if isinstance(value, basestring) or not isiterable(value):
+            if isinstance(value, string_types) or not isiterable(value):
                 value = itertools.repeat(value, len(indices))
-            for idx, val in itertools.izip(indices, value):
+            for idx, val in zip(indices, value):
                 self[idx] = val
             return True
         return False
@@ -2243,7 +2278,7 @@ class _HeaderCommentaryCards(_CardAccessor):
         elif not isinstance(idx, int):
             raise ValueError('%s index must be an integer' % self._keyword)
 
-        idx = range(*self._indices)[idx]
+        idx = list(range(*self._indices))[idx]
         return self._header[(self._keyword, idx)]
 
     def __setitem__(self, item, value):
@@ -2261,20 +2296,6 @@ class _HeaderCommentaryCards(_CardAccessor):
         self._header[(self._keyword, item)] = value
 
 
-def _is_astropy_internal():
-    """
-    Returns True if the stack frame this is called from is in code internal to
-    the the astropy package.
-
-    This is used in a few places where hacks are employed for backwards
-    compatibility with the old header API, but where we want to avoid using
-    those hacks internally.
-    """
-
-    calling_mod = inspect.getmodule(sys._getframe(2))
-    return calling_mod and calling_mod.__name__.startswith('astropy.')
-
-
 def _block_size(sep):
     """
     Determine the size of a FITS header block if a non-blank separator is used
