@@ -2,7 +2,6 @@
 
 import sys
 import warnings
-import sys
 
 import numpy as np
 
@@ -42,7 +41,7 @@ class TestChecksumFunctions(FitsTestCase):
         hdul.close()
 
     def test_image_create(self):
-        n = np.arange(100)
+        n = np.arange(100, dtype=np.int64)
         hdu = fits.PrimaryHDU(n)
         hdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
@@ -57,7 +56,7 @@ class TestChecksumFunctions(FitsTestCase):
                 assert hdul[0].header['DATASUM'] == '4950'
 
     def test_nonstandard_checksum(self):
-        hdu = fits.PrimaryHDU(np.arange(10.0 ** 6))
+        hdu = fits.PrimaryHDU(np.arange(10.0 ** 6, dtype=np.float64))
         hdu.writeto(self.temp('tmp.fits'), clobber=True,
                     checksum='nonstandard')
         del hdu
@@ -83,11 +82,52 @@ class TestChecksumFunctions(FitsTestCase):
                 assert 'DATASUM' in hdul1[0].header
                 assert hdul1[0].header['DATASUM'] == '1891563534'
 
+    def test_scaled_data_auto_rescale(self):
+        """
+        Regression test for
+        https://github.com/astropy/astropy/issues/3883#issuecomment-115122647
+
+        Ensure that when scaled data is automatically rescaled on
+        opening/writing a file that the checksum and datasum are computed for
+        the rescaled array.
+        """
+
+        with fits.open(self.data('scale.fits')) as hdul:
+            # Write out a copy of the data with the rescaling applied
+            hdul.writeto(self.temp('rescaled.fits'))
+
+        # Reopen the new file and save it back again with a checksum
+        with fits.open(self.temp('rescaled.fits')) as hdul:
+            hdul.writeto(self.temp('rescaled2.fits'), clobber=True,
+                         checksum=True)
+
+        # Now do like in the first writeto but use checksum immediately
+        with fits.open(self.data('scale.fits')) as hdul:
+            hdul.writeto(self.temp('rescaled3.fits'), checksum=True)
+
+        # Also don't rescale the data but add a checksum
+        with fits.open(self.data('scale.fits'),
+                       do_not_scale_image_data=True) as hdul:
+            hdul.writeto(self.temp('scaled.fits'), checksum=True)
+
+        # Must used nested with statements to support older Python versions
+        # (but contextlib.nested is not available in newer Pythons :(
+        with fits.open(self.temp('rescaled2.fits')) as hdul1:
+            with fits.open(self.temp('rescaled3.fits')) as hdul2:
+                with fits.open(self.temp('scaled.fits')) as hdul3:
+                    hdr1 = hdul1[0].header
+                    hdr2 = hdul2[0].header
+                    hdr3 = hdul3[0].header
+                    assert hdr1['DATASUM'] == hdr2['DATASUM']
+                    assert hdr1['CHECKSUM'] == hdr2['CHECKSUM']
+                    assert hdr1['DATASUM'] != hdr3['DATASUM']
+                    assert hdr1['CHECKSUM'] != hdr3['CHECKSUM']
+
     def test_uint16_data(self):
         checksums = [
-            ('aDcXaCcXaCcXaCcX', '0'), ('84DJ83CH83CH83CH', '1746888714'),
+            ('aDcXaCcXaCcXaCcX', '0'), ('oYiGqXi9oXiEoXi9', '1746888714'),
             ('VhqQWZoQVfoQVZoQ', '0'), ('4cPp5aOn4aOn4aOn', '0'),
-            ('97ZYI7WV97WVG7WV', '1756785133'), ('UhqdUZnbUfnbUZnb', '0'),
+            ('8aCN8X9N8aAN8W9N', '1756785133'), ('UhqdUZnbUfnbUZnb', '0'),
             ('4cQJ5aN94aNG4aN9', '0')]
         with fits.open(self.data('o4sp040b0_raw.fits'), uint=True) as hdul:
             hdul.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
@@ -109,7 +149,7 @@ class TestChecksumFunctions(FitsTestCase):
         imdata.shape = (10, 1, 1, 2, 5)
         pdata1 = np.arange(10) + 0.1
         pdata2 = 42
-        x = fits.hdu.groups.GroupData(imdata, parnames=['abc', 'xyz'],
+        x = fits.hdu.groups.GroupData(imdata, parnames=[str('abc'), str('xyz')],
                                       pardata=[pdata1, pdata2], bitpix=-32)
         hdu = fits.GroupsHDU(x)
         hdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
@@ -126,7 +166,7 @@ class TestChecksumFunctions(FitsTestCase):
         col1 = fits.Column(name='target', format='20A', array=a1)
         col2 = fits.Column(name='V_mag', format='E', array=a2)
         cols = fits.ColDefs([col1, col2])
-        tbhdu = fits.new_table(cols)
+        tbhdu = fits.BinTableHDU.from_columns(cols)
         tbhdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
             assert comparerecords(tbhdu.data, hdul[1].data)
@@ -144,7 +184,7 @@ class TestChecksumFunctions(FitsTestCase):
                          array=np.array([[45.0, 56], np.array([11, 12, 13])],
                          'O'))
         c2 = fits.Column(name='xyz', format='2I', array=[[11, 3], [12, 4]])
-        tbhdu = fits.new_table([c1, c2])
+        tbhdu = fits.BinTableHDU.from_columns([c1, c2])
         tbhdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
             assert comparerecords(tbhdu.data, hdul[1].data)
@@ -166,8 +206,8 @@ class TestChecksumFunctions(FitsTestCase):
         c2 = fits.Column(name='def', format='D', array=r1, bscale=2.3,
                          bzero=0.6)
         c3 = fits.Column(name='t1', format='I', array=[91, 92, 93])
-        x = fits.ColDefs([c1, c2, c3], tbtype='TableHDU')
-        hdu = fits.new_table(x, tbtype='TableHDU')
+        x = fits.ColDefs([c1, c2, c3])
+        hdu = fits.TableHDU.from_columns(x)
         hdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
             assert comparerecords(hdu.data, hdul[1].data)
@@ -337,7 +377,7 @@ class TestChecksumFunctions(FitsTestCase):
 
     def test_open_update_mode_preserve_checksum(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/148 where
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/148 where
         checksums are being removed from headers when a file is opened in
         update mode, even though no changes were made to the file.
         """
@@ -353,11 +393,11 @@ class TestChecksumFunctions(FitsTestCase):
         with fits.open(self.temp('checksum.fits')) as hdul:
             assert 'CHECKSUM' in hdul[1].header
             assert 'DATASUM' in hdul[1].header
-            assert (data == hdul[1].data).all()
+            assert comparerecords(data, hdul[1].data)
 
     def test_open_update_mode_update_checksum(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/148, part
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/148, part
         2.  This ensures that if a file contains a checksum, the checksum is
         updated when changes are saved to the file, even if the file was opened
         with the default of checksum=False.
