@@ -4,6 +4,7 @@ from __future__ import division, with_statement
 
 import math
 import os
+import re
 import time
 import warnings
 
@@ -12,12 +13,11 @@ import numpy as np
 from ....io import fits
 from ....utils.exceptions import (AstropyDeprecationWarning,
                                   AstropyPendingDeprecationWarning)
-from ....tests.helper import pytest, raises, catch_warnings
+from ....tests.helper import pytest, raises, catch_warnings, ignore_warnings
 from ..hdu.compressed import SUBTRACTIVE_DITHER_1, DITHER_SEED_CHECKSUM
 from .test_table import comparerecords
 
 from . import FitsTestCase
-from .util import ignore_warnings
 
 
 class TestImageFunctions(FitsTestCase):
@@ -627,6 +627,65 @@ class TestImageFunctions(FitsTestCase):
                 assert 'BZERO' in new_uint_hdu.header
                 assert new_uint_hdu.header['BZERO'] == (2 ** (int_size - 1))
 
+    @pytest.mark.parametrize(('from_file'), (False, True))
+    @pytest.mark.parametrize(('do_not_scale'), (False,))
+    def test_uint_header_keywords_removed_after_bitpix_change(self,
+                                                              from_file,
+                                                              do_not_scale):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/4974
+
+        BZERO/BSCALE should be removed if data is converted to a floating
+        point type.
+
+        Currently excluding the case where do_not_scale_image_data=True
+        because it is not clear what the expectation should be.
+        """
+
+        arr = np.zeros(100, dtype='uint16')
+
+        if from_file:
+            # To generate the proper input file we always want to scale the
+            # data before writing it...otherwise when we open it will be
+            # regular (signed) int data.
+            tmp_uint = fits.PrimaryHDU(arr)
+            filename = 'unsigned_int.fits'
+            tmp_uint.writeto(self.temp(filename))
+            f = fits.open(self.temp(filename),
+                          do_not_scale_image_data=do_not_scale)
+
+            uint_hdu = f[0]
+            # Force a read before we close.
+            _ = uint_hdu.data
+        else:
+            uint_hdu = fits.PrimaryHDU(arr,
+                                       do_not_scale_image_data=do_not_scale)
+
+        # Make sure appropriate keywords are in the header. See
+        # https://github.com/astropy/astropy/pull/3916#issuecomment-122414532
+        # for discussion.
+        assert 'BSCALE' in uint_hdu.header
+        assert 'BZERO' in uint_hdu.header
+        assert uint_hdu.header['BSCALE'] == 1
+        assert uint_hdu.header['BZERO'] == 32768
+
+        # Convert data to floating point...
+        uint_hdu.data = uint_hdu.data * 1.0
+
+        # ...bitpix should be negative.
+        assert uint_hdu.header['BITPIX'] < 0
+
+        # BSCALE and BZERO should NOT be in header any more.
+        assert 'BSCALE' not in uint_hdu.header
+        assert 'BZERO' not in uint_hdu.header
+
+        # This is the main test...the data values should round trip
+        # as zero.
+        filename = 'test_uint_to_float.fits'
+        uint_hdu.writeto(self.temp(filename))
+        with fits.open(self.temp(filename)) as hdul:
+            assert (hdul[0].data == 0).all()
+
     def test_blanks(self):
         """Test image data with blank spots in it (which should show up as
         NaNs in the data array.
@@ -688,7 +747,7 @@ class TestImageFunctions(FitsTestCase):
         hdu = fits.PrimaryHDU(data=arr)
         hdu.scale('int16', bscale=1.23)
 
-        # Creating data that uses BLANK is currently kludgy--a speparate issue
+        # Creating data that uses BLANK is currently kludgy--a separate issue
         # TODO: Rewrite this test when scaling with blank support is better
         # supported
 
@@ -1266,8 +1325,8 @@ class TestCompressedImage(FitsTestCase):
 
         with fits.open(self.temp('test.fits'),
                        disable_image_compression=True) as h:
-            assert h[1].header['TFORM1'] == '1PB(30)'
-            assert h[1].header['TFORM2'] == '1PB(359)'
+            assert re.match(r'^1PB\(\d+\)$', h[1].header['TFORM1'])
+            assert re.match(r'^1PB\(\d+\)$', h[1].header['TFORM2'])
 
     def test_compression_update_header(self):
         """Regression test for
@@ -1494,3 +1553,12 @@ class TestCompressedImage(FitsTestCase):
             # There's no good reason to have a duplicate keyword, but
             # technically it isn't invalid either :/
             assert hdul[1]._header.count('ZTENSION') == 2
+
+
+def test_set_data():
+    """
+    Test data assignment - issue #5087
+    """
+    im = fits.ImageHDU()
+    ar = np.arange(12)
+    im.data = ar
