@@ -14,10 +14,8 @@ from ....extern.six.moves import xrange
 # STDLIB
 import difflib
 import io
-import os
-import shutil
 import sys
-import tempfile
+import gzip
 
 # THIRD-PARTY
 from numpy.testing import assert_array_equal
@@ -30,7 +28,13 @@ from ..exceptions import VOTableSpecError, VOWarning
 from ..xmlutil import validate_schema
 from ....utils.data import get_pkg_data_filename, get_pkg_data_filenames
 from ....tests.helper import pytest, raises, catch_warnings
-from ....utils.compat import gzip
+
+try:
+    import pathlib
+except ImportError:
+    HAS_PATHLIB = False
+else:
+    HAS_PATHLIB = True
 
 # Determine the kind of float formatting in this build of Python
 if hasattr(sys, 'float_repr_style'):
@@ -38,18 +42,6 @@ if hasattr(sys, 'float_repr_style'):
 else:
     legacy_float_repr = sys.platform.startswith('win')
 
-join = os.path.join
-
-
-# Update this to use py.test's tmpdir functionality
-def setup_module():
-    global TMP_DIR
-    TMP_DIR = tempfile.mkdtemp()
-
-
-def teardown_module():
-    shutil.rmtree(TMP_DIR)
-
 
 def assert_validate_schema(filename, version):
     if sys.platform.startswith('win'):
@@ -88,7 +80,7 @@ def test_parse_single_table3():
         table_number=3, pedantic=False)
 
 
-def _test_regression(_python_based=False, binary_mode=1):
+def _test_regression(tmpdir, _python_based=False, binary_mode=1):
     # Read the VOTABLE
     votable = parse(
         get_pkg_data_filename('data/regression.xml'),
@@ -135,11 +127,10 @@ def _test_regression(_python_based=False, binary_mode=1):
         dtypes = new_dtypes
     assert table.array.dtype == dtypes
 
-    votable.to_xml(join(TMP_DIR, "regression.tabledata.xml"),
+    votable.to_xml(str(tmpdir.join("regression.tabledata.xml")),
                    _debug_python_based_parser=_python_based)
-    assert_validate_schema(
-        join(TMP_DIR, "regression.tabledata.xml"),
-        votable.version)
+    assert_validate_schema(str(tmpdir.join("regression.tabledata.xml")),
+                           votable.version)
 
     if binary_mode == 1:
         votable.get_first_table().format = 'binary'
@@ -150,22 +141,20 @@ def _test_regression(_python_based=False, binary_mode=1):
         votable.version = '1.3'
 
     # Also try passing a file handle
-    with open(join(TMP_DIR, "regression.binary.xml"), "wb") as fd:
+    with open(str(tmpdir.join("regression.binary.xml")), "wb") as fd:
         votable.to_xml(fd, _debug_python_based_parser=_python_based)
-    assert_validate_schema(
-        join(TMP_DIR, "regression.binary.xml"),
-        votable.version)
+    assert_validate_schema(str(tmpdir.join("regression.binary.xml")),
+                           votable.version)
     # Also try passing a file handle
-    with open(join(TMP_DIR, "regression.binary.xml"), "rb") as fd:
+    with open(str(tmpdir.join("regression.binary.xml")), "rb") as fd:
         votable2 = parse(fd, pedantic=False,
                          _debug_python_based_parser=_python_based)
     votable2.get_first_table().format = 'tabledata'
-    votable2.to_xml(join(TMP_DIR, "regression.bin.tabledata.xml"),
+    votable2.to_xml(str(tmpdir.join("regression.bin.tabledata.xml")),
                     _astropy_version="testing",
                     _debug_python_based_parser=_python_based)
-    assert_validate_schema(
-        join(TMP_DIR, "regression.bin.tabledata.xml"),
-        votable.version)
+    assert_validate_schema(str(tmpdir.join("regression.bin.tabledata.xml")),
+                           votable.version)
 
     with io.open(
         get_pkg_data_filename(
@@ -173,9 +162,8 @@ def _test_regression(_python_based=False, binary_mode=1):
                 votable.version)),
         'rt', encoding='utf-8') as fd:
         truth = fd.readlines()
-    with io.open(
-        join(TMP_DIR, "regression.bin.tabledata.xml"),
-        'rt', encoding='utf-8') as fd:
+    with io.open(str(tmpdir.join("regression.bin.tabledata.xml")),
+                 'rt', encoding='utf-8') as fd:
         output = fd.readlines()
 
     # If the lines happen to be different, print a diff
@@ -190,11 +178,11 @@ def _test_regression(_python_based=False, binary_mode=1):
 
     # Test implicit gzip saving
     votable2.to_xml(
-        join(TMP_DIR, "regression.bin.tabledata.xml.gz"),
+        str(tmpdir.join("regression.bin.tabledata.xml.gz")),
         _astropy_version="testing",
         _debug_python_based_parser=_python_based)
     with gzip.GzipFile(
-        join(TMP_DIR, "regression.bin.tabledata.xml.gz"), 'rb') as gzfd:
+        str(tmpdir.join("regression.bin.tabledata.xml.gz")), 'rb') as gzfd:
         output = gzfd.readlines()
     output = [x.decode('utf-8').rstrip() for x in output]
     truth = [x.rstrip() for x in truth]
@@ -203,18 +191,18 @@ def _test_regression(_python_based=False, binary_mode=1):
 
 
 @pytest.mark.xfail(str('legacy_float_repr'))
-def test_regression():
-    _test_regression(False)
+def test_regression(tmpdir):
+    _test_regression(tmpdir, False)
 
 
 @pytest.mark.xfail(str('legacy_float_repr'))
-def test_regression_python_based_parser():
-    _test_regression(True)
+def test_regression_python_based_parser(tmpdir):
+    _test_regression(tmpdir, True)
 
 
 @pytest.mark.xfail(str('legacy_float_repr'))
-def test_regression_binary2():
-    _test_regression(False, 2)
+def test_regression_binary2(tmpdir):
+    _test_regression(tmpdir, False, 2)
 
 
 class TestFixups:
@@ -611,15 +599,32 @@ class TestParse:
         assert fields[0].name == "int"
         assert fields[0].values.min == -1000
 
+    def test_get_info_by_id(self):
+        info = self.votable.get_info_by_id('QUERY_STATUS')
+        assert info.value == 'OK'
+
+        if self.votable.version != '1.1':
+            info = self.votable.get_info_by_id("ErrorInfo")
+            assert info.value == "One might expect to find some INFO here, too..."
+
+    def test_repr(self):
+        assert '3 tables' in repr(self.votable)
+        assert repr(list(self.votable.iter_fields_and_params())[0]) == \
+            '<PARAM ID="awesome" arraysize="*" datatype="float" name="INPUT" unit="deg" value="[0.0 0.0]"/>'
+        # Smoke test
+        repr(list(self.votable.iter_groups()))
+
 
 class TestThroughTableData(TestParse):
     def setup_class(self):
         votable = parse(
             get_pkg_data_filename('data/regression.xml'),
             pedantic=False)
-        votable.to_xml(join(TMP_DIR, "test_through_tabledata.xml"))
-        self.votable = parse(join(TMP_DIR, "test_through_tabledata.xml"),
-                           pedantic=False)
+
+        self.xmlout = bio = io.BytesIO()
+        votable.to_xml(bio)
+        bio.seek(0)
+        self.votable = parse(bio, pedantic=False)
         self.table = self.votable.get_first_table()
         self.array = self.table.array
         self.mask = self.table.array.mask
@@ -634,9 +639,13 @@ class TestThroughTableData(TestParse):
     def test_bit_array2_mask(self):
         assert not np.any(self.mask['bitarray2'])
 
-    def test_schema(self):
-        assert_validate_schema(
-            join(TMP_DIR, "test_through_tabledata.xml"), '1.1')
+    def test_schema(self, tmpdir):
+        # have to use an actual file because assert_validate_schema only works
+        # on filenames, not file-like objects
+        fn = str(tmpdir.join("test_through_tabledata.xml"))
+        with open(fn, 'wb') as f:
+            f.write(self.xmlout.getvalue())
+        assert_validate_schema(fn, '1.1')
 
 
 class TestThroughBinary(TestParse):
@@ -645,9 +654,12 @@ class TestThroughBinary(TestParse):
             get_pkg_data_filename('data/regression.xml'),
             pedantic=False)
         votable.get_first_table().format = 'binary'
-        votable.to_xml(join(TMP_DIR, "test_through_binary.xml"))
-        self.votable = parse(join(TMP_DIR, "test_through_binary.xml"),
-                           pedantic=False)
+
+        self.xmlout = bio = io.BytesIO()
+        votable.to_xml(bio)
+        bio.seek(0)
+        self.votable = parse(bio, pedantic=False)
+
         self.table = self.votable.get_first_table()
         self.array = self.table.array
         self.mask = self.table.array.mask
@@ -674,9 +686,11 @@ class TestThroughBinary2(TestParse):
         votable.get_first_table()._config['version_1_3_or_later'] = True
         votable.get_first_table().format = 'binary2'
 
-        votable.to_xml(join(TMP_DIR, "test_through_binary2.xml"))
-        self.votable = parse(join(TMP_DIR, "test_through_binary2.xml"),
-                           pedantic=False)
+        self.xmlout = bio = io.BytesIO()
+        votable.to_xml(bio)
+        bio.seek(0)
+        self.votable = parse(bio, pedantic=False)
+
         self.table = self.votable.get_first_table()
         self.array = self.table.array
         self.mask = self.table.array.mask
@@ -723,7 +737,7 @@ def test_open_files():
     def test_file(filename):
         parse(filename, pedantic=False)
 
-    for filename in get_pkg_data_filenames('data', '*.xml'):
+    for filename in get_pkg_data_filenames('data', pattern='*.xml'):
         if filename.endswith('custom_datatype.xml'):
             continue
         yield test_file, filename
@@ -736,7 +750,7 @@ def test_too_many_columns():
         pedantic=False)
 
 
-def test_build_from_scratch():
+def test_build_from_scratch(tmpdir):
     # Create a new VOTable file...
     votable = tree.VOTableFile()
 
@@ -763,9 +777,9 @@ def test_build_from_scratch():
 
     # Now write the whole thing to a file.
     # Note, we have to use the top-level votable file object
-    votable.to_xml(os.path.join(TMP_DIR, "new_votable.xml"))
+    votable.to_xml(str(tmpdir.join("new_votable.xml")))
 
-    votable = parse(os.path.join(TMP_DIR, "new_votable.xml"))
+    votable = parse(str(tmpdir.join("new_votable.xml")))
 
     table = votable.get_first_table()
     assert_array_equal(
@@ -775,13 +789,20 @@ def test_build_from_scratch():
                                            (str('matrix'), str('?'), (2, 2))]))
 
 
-def test_validate():
+def test_validate(test_path_object=False):
+    """
+    test_path_object is needed for test below ``test_validate_path_object``
+    so that file could be passed as pathlib.Path object.
+    """
     output = io.StringIO()
+    fpath = get_pkg_data_filename('data/regression.xml')
+    if test_path_object:
+        fpath = pathlib.Path(fpath)
 
     # We can't test xmllint, because we can't rely on it being on the
     # user's machine.
     with catch_warnings():
-        result = validate(get_pkg_data_filename('data/regression.xml'),
+        result = validate(fpath,
                           output, xmllint=False)
 
     assert result == False
@@ -813,18 +834,27 @@ def test_validate():
     assert truth == output
 
 
-def test_gzip_filehandles():
+@pytest.mark.skipif('not HAS_PATHLIB')
+def test_validate_path_object():
+    """
+    Validating when source is passed as path object. (#4412)
+    """
+    test_validate(test_path_object=True)
+
+
+def test_gzip_filehandles(tmpdir):
     votable = parse(
         get_pkg_data_filename('data/regression.xml'),
         pedantic=False)
 
-    with open(join(TMP_DIR, "regression.compressed.xml"), 'wb') as fd:
+
+    with open(str(tmpdir.join("regression.compressed.xml")), 'wb') as fd:
         votable.to_xml(
             fd,
             compressed=True,
             _astropy_version="testing")
 
-    with open(join(TMP_DIR, "regression.compressed.xml"), 'rb') as fd:
+    with open(str(tmpdir.join("regression.compressed.xml")), 'rb') as fd:
         votable = parse(
             fd,
             pedantic=False)
