@@ -7,7 +7,7 @@ Creates a common namespace for all pre-defined models.
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-from .core import custom_model
+from .core import custom_model  # pylint: disable=W0611
 from .mappings import *
 from .projections import *
 from .rotations import *
@@ -58,9 +58,9 @@ MODELS_WITH_CONSTRAINTS = [
     Const1D, Const2D, Ellipse2D, Disk2D,
     Gaussian1D, GaussianAbsorption1D, Gaussian2D,
     Linear1D, Lorentz1D, MexicanHat1D, MexicanHat2D,
-    PowerLaw1D, Sine1D, Trapezoid1D, TrapezoidDisk2D,
+    PowerLaw1D, Sersic1D, Sersic2D, Sine1D, Trapezoid1D, TrapezoidDisk2D,
     Chebyshev1D, Chebyshev2D, Legendre2D, Legendre1D,
-    Polynomial1D, Polynomial2D
+    Polynomial1D, Polynomial2D, Voigt1D
 ]
 
 
