@@ -15,12 +15,12 @@ from copy import deepcopy
 import warnings
 import collections
 import itertools
+from collections import OrderedDict, Counter
 
 import numpy as np
 from numpy import ma
 
-from ..utils import OrderedDict, metadata
-from .column import col_getattr, col_setattr, _col_update_attrs_from
+from ..utils import metadata
 
 from . import _np_utils
 from .np_utils import fix_column_name, TableMergeError
@@ -44,14 +44,13 @@ def _merge_col_meta(out, tables, col_name_map, idx_left=0, idx_right=1,
     for out_col in six.itervalues(out.columns):
         for idx_table, table in enumerate(tables):
             left_col = out_col
-            right_name = col_name_map[col_getattr(out_col, 'name')][idx_table]
+            right_name = col_name_map[out_col.info.name][idx_table]
 
             if right_name:
                 right_col = table[right_name]
-                col_setattr(out_col, 'meta',
-                            metadata.merge(col_getattr(left_col, 'meta', {}),
-                                           col_getattr(right_col, 'meta', {}),
-                                           metadata_conflicts=metadata_conflicts))
+                out_col.info.meta = metadata.merge(left_col.info.meta or {},
+                                                   right_col.info.meta or {},
+                                                   metadata_conflicts=metadata_conflicts)
                 for attr in attrs:
 
                     # Pick the metadata item that is not None, or they are both
@@ -73,13 +72,13 @@ def _merge_col_meta(out, tables, col_name_map, idx_left=0, idx_right=1,
                         if metadata_conflicts == 'warn':
                             warnings.warn("In merged column '{0}' the '{1}' attribute does not match "
                                           "({2} != {3}).  Using {3} for merged output"
-                                          .format(col_getattr(out_col, 'name'), attr,
+                                          .format(out_col.info.name, attr,
                                                   left_attr, right_attr),
                                           metadata.MergeConflictWarning)
                         elif metadata_conflicts == 'error':
                             raise metadata.MergeConflictError(
                                 'In merged column {0!r} the {1!r} attribute does not match '
-                                '({2} != {3})'.format(col_getattr(out_col, 'name'), attr,
+                                '({2} != {3})'.format(out_col.info.name, attr,
                                                       left_attr, right_attr))
                         elif metadata_conflicts != 'silent':
                             raise ValueError('metadata_conflicts argument must be one of "silent",'
@@ -199,7 +198,7 @@ def vstack(tables, join_type='outer', metadata_conflicts='warn'):
 
     A ``join_type`` of 'exact' means that the tables must all have exactly
     the same column names (though the order can vary).  If ``join_type``
-    is 'inner' then the intersection of common columns will be output.
+    is 'inner' then the intersection of common columns will be the output.
     A value of 'outer' (default) means the output will have the union of
     all columns, with table values being masked where no common values are
     available.
@@ -209,7 +208,7 @@ def vstack(tables, join_type='outer', metadata_conflicts='warn'):
     tables : Table or list of Table objects
         Table(s) to stack along rows (vertically) with the current table
     join_type : str
-        Join type ('inner' | 'exact' | 'outer'), default is 'exact'
+        Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     metadata_conflicts : str
         How to proceed with metadata conflicts. This should be one of:
             * ``'silent'``: silently pick the last conflicting meta-data value
@@ -268,8 +267,8 @@ def hstack(tables, join_type='outer',
 
     A ``join_type`` of 'exact' means that the tables must all
     have exactly the same number of rows.  If ``join_type`` is 'inner' then
-    the intersection of rows will be output.  A value of 'outer' (default) means
-    the output will have the union of all rows, with table values being
+    the intersection of rows will be the output.  A value of 'outer' (default)
+    means the output will have the union of all rows, with table values being
     masked where no common values are available.
 
     Parameters
@@ -332,7 +331,7 @@ def hstack(tables, join_type='outer',
     return out
 
 
-def unique(input_table, keys=None, silent=False):
+def unique(input_table, keys=None, silent=False, keep='first'):
     """
     Returns the unique rows of a table.
 
@@ -340,56 +339,127 @@ def unique(input_table, keys=None, silent=False):
     ----------
 
     input_table : `~astropy.table.Table` object or a value that
-    will initialize a `~astropy.table.Table` object
-        Input table.
+        will initialize a `~astropy.table.Table` object
     keys : str or list of str
-        Name(s) of column(s) used to unique rows.
+        Name(s) of column(s) used to create unique rows.
         Default is to use all columns.
+    keep : one of 'first', 'last' or 'none'
+        Whether to keep the first or last row for each set of
+        duplicates. If 'none', all rows that are duplicate are
+        removed, leaving only rows that are already unique in
+        the input.
+        Default is 'first'.
     silent : boolean
-        If `True` masked value column(s) are silently removed from
-        ``keys``. If `False` an exception is raised when ``keys`` contains
-        masked value column(s).
+        If `True`, masked value column(s) are silently removed from
+        ``keys``. If `False`, an exception is raised when ``keys``
+        contains masked value column(s).
         Default is `False`.
 
     Returns
     -------
     unique_table : `~astropy.table.Table` object
-        Table containing only the unique rays of ``input_table``.
+        New table containing only the unique rows of ``input_table``.
+
+    Examples
+    --------
+    >>> from astropy.table import Table
+    >>> import numpy as np
+    >>> table = Table(data=[[1,2,3,2,3,3],
+    ... [2,3,4,5,4,6],
+    ... [3,4,5,6,7,8]],
+    ... names=['col1', 'col2', 'col3'],
+    ... dtype=[np.int32, np.int32, np.int32])
+    >>> table
+    <Table length=6>
+     col1  col2  col3
+    int32 int32 int32
+    ----- ----- -----
+        1     2     3
+        2     3     4
+        3     4     5
+        2     5     6
+        3     4     7
+        3     6     8
+    >>> unique(table, keys='col1')
+    <Table length=3>
+     col1  col2  col3
+    int32 int32 int32
+    ----- ----- -----
+        1     2     3
+        2     3     4
+        3     4     5
+    >>> unique(table, keys=['col1'], keep='last')
+    <Table length=3>
+     col1  col2  col3
+    int32 int32 int32
+    ----- ----- -----
+        1     2     3
+        2     5     6
+        3     6     8
+    >>> unique(table, keys=['col1', 'col2'])
+    <Table length=5>
+     col1  col2  col3
+    int32 int32 int32
+    ----- ----- -----
+        1     2     3
+        2     3     4
+        2     5     6
+        3     4     5
+        3     6     8
+    >>> unique(table, keys=['col1', 'col2'], keep='none')
+    <Table length=4>
+     col1  col2  col3
+    int32 int32 int32
+    ----- ----- -----
+        1     2     3
+        2     3     4
+        2     5     6
+        3     6     8
+    >>> unique(table, keys=['col1'], keep='none')
+    <Table length=1>
+     col1  col2  col3
+    int32 int32 int32
+    ----- ----- -----
+        1     2     3
 
     """
 
+    if keep not in ('first', 'last', 'none'):
+        raise ValueError("'keep' should be one of 'first', 'last', 'none'")
+
+    if isinstance(keys, six.string_types):
+        keys = [keys]
     if keys is None:
         keys = input_table.colnames
+    else:
+        if len(set(keys)) != len(keys):
+            raise ValueError("duplicate key names")
 
     if input_table.masked:
-        if isinstance(keys, six.string_types):
-            keys = [keys, ]
-        for i, key in enumerate(keys):
+        nkeys = 0
+        for key in keys[:]:
             if np.any(input_table[key].mask):
                 if not silent:
-                    raise ValueError("Cannot unique masked value key columns, "
-                                     "remove column '{0}' from keys and rerun "
-                                     "unique.".format(key))
-                del keys[i]
+                    raise ValueError(
+                        "cannot use columns with masked values as keys; "
+                        "remove column '{0}' from keys and rerun "
+                        "unique()".format(key))
+                del keys[keys.index(key)]
         if len(keys) == 0:
-            raise ValueError("No column remained in ``keys``, unique cannot "
-                             "work with masked value key columns.")
+            raise ValueError("no column remained in ``keys``; "
+                             "unique() cannot work with masked value "
+                             "key columns")
 
     grouped_table = input_table.group_by(keys)
-    unique_table = grouped_table[grouped_table.groups.indices[:-1]]
+    indices = grouped_table.groups.indices
+    if keep == 'first':
+        indices = indices[:-1]
+    elif keep == 'last':
+        indices = indices[1:] - 1
+    else:
+        indices = indices[:-1][np.diff(indices) == 1]
 
-    return unique_table
-
-
-def _counter(iterable):
-    """
-    Count instances of each unique value in ``iterable``.  Returns a dict
-    with the counts.  Would use collections.Counter but this isn't available in 2.6.
-    """
-    counts = collections.defaultdict(int)
-    for val in iterable:
-        counts[val] += 1
-    return counts
+    return grouped_table[indices]
 
 
 def get_col_name_map(arrays, common_names, uniq_col_name='{col_name}_{table_name}',
@@ -435,7 +505,7 @@ def get_col_name_map(arrays, common_names, uniq_col_name='{col_name}_{table_name
             col_name_map[out_name][idx] = name
 
     # Check for duplicate output column names
-    col_name_count = _counter(col_name_list)
+    col_name_count = Counter(col_name_list)
     repeated_names = [name for name, count in six.iteritems(col_name_count) if count > 1]
     if repeated_names:
         raise TableMergeError('Merging column names resulted in duplicates: {0}.  '
@@ -492,31 +562,14 @@ def common_dtype(cols):
     Only allow columns within the following fundamental numpy data types:
     np.bool_, np.object_, np.number, np.character, np.void
     """
-    def dtype(col):
-        return getattr(col, 'dtype', np.dtype('O'))
-
-    np_types = (np.bool_, np.object_, np.number, np.character, np.void)
-    uniq_types = set(tuple(issubclass(dtype(col).type, np_type) for np_type in np_types)
-                     for col in cols)
-    if len(uniq_types) > 1:
-        # Embed into the exception the actual list of incompatible types.
-        incompat_types = [dtype(col).name for col in cols]
+    try:
+        return metadata.common_dtype(cols)
+    except metadata.MergeConflictError as err:
         tme = TableMergeError('Columns have incompatible types {0}'
-                              .format(incompat_types))
-        tme._incompat_types = incompat_types
+                              .format(err._incompat_types))
+        tme._incompat_types = err._incompat_types
         raise tme
 
-    arrs = [np.empty(1, dtype=dtype(col)) for col in cols]
-
-    # For string-type arrays need to explicitly fill in non-zero
-    # values or the final arr_common = .. step is unpredictable.
-    for arr in arrs:
-        if arr.dtype.kind in ('S', 'U'):
-            arr[0] = '0' * arr.itemsize
-
-    arr_common = np.array([arr[0] for arr in arrs])
-    return arr_common.dtype.str
-
 
 def _join(left, right, keys=None, join_type='inner',
          uniq_col_name='{col_name}_{table_name}',
@@ -634,13 +687,17 @@ def _join(left, right, keys=None, join_type='inner',
 
         # Finally add the joined column to the output table.
         out[out_name] = array[name][array_out]
-        _col_update_attrs_from(out[out_name], array[name])
 
         # If the output table is masked then set the output column masking
         # accordingly.  Check for columns that don't support a mask attribute.
         if masked:
+            # array_mask is 1-d corresponding to length of output column.  We need
+            # make it have the correct shape for broadcasting, i.e. (length, 1, 1, ..).
+            # Mixin columns might not have ndim attribute so use len(col.shape).
+            array_mask.shape = (out[out_name].shape[0],) + (1,) * (len(out[out_name].shape) - 1)
+
             if array.masked:
-                array_mask = array_mask | array[name].mask.take(array_out)
+                array_mask = array_mask | array[name].mask[array_out]
             try:
                 out[out_name].mask[:] = array_mask
             except ValueError:
@@ -655,14 +712,14 @@ def _join(left, right, keys=None, join_type='inner',
     return out
 
 
-def _vstack(arrays, join_type='inner', col_name_map=None):
+def _vstack(arrays, join_type='outer', col_name_map=None):
     """
     Stack Tables vertically (by rows)
 
     A ``join_type`` of 'exact' (default) means that the arrays must all
     have exactly the same column names (though the order can vary).  If
     ``join_type`` is 'inner' then the intersection of common columns will
-    be output.  A value of 'outer' means the output will have the union of
+    be the output.  A value of 'outer' means the output will have the union of
     all columns, with array values being masked where no common values are
     available.
 
@@ -671,7 +728,7 @@ def _vstack(arrays, join_type='inner', col_name_map=None):
     arrays : list of Tables
         Tables to stack by rows (vertically)
     join_type : str
-        Join type ('inner' | 'exact' | 'outer'), default is 'exact'
+        Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     col_name_map : empty dict or None
         If passed as a dict then it will be updated in-place with the
         mapping of output to input column names.
@@ -754,14 +811,14 @@ def _vstack(arrays, join_type='inner', col_name_map=None):
     return out
 
 
-def _hstack(arrays, join_type='exact', uniq_col_name='{col_name}_{table_name}',
+def _hstack(arrays, join_type='outer', uniq_col_name='{col_name}_{table_name}',
            table_names=None, col_name_map=None):
     """
     Stack tables horizontally (by columns)
 
     A ``join_type`` of 'exact' (default) means that the arrays must all
     have exactly the same number of rows.  If ``join_type`` is 'inner' then
-    the intersection of rows will be output.  A value of 'outer' means
+    the intersection of rows will be the output.  A value of 'outer' means
     the output will have the union of all rows, with array values being
     masked where no common values are available.
 
@@ -770,7 +827,7 @@ def _hstack(arrays, join_type='exact', uniq_col_name='{col_name}_{table_name}',
     arrays : List of tables
         Tables to stack by columns (horizontally)
     join_type : str
-        Join type ('inner' | 'exact' | 'outer'), default is 'exact'
+        Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     uniq_col_name : str or None
         String generate a unique output column name in case of a conflict.
         The default is '{col_name}_{table_name}'.
@@ -844,8 +901,6 @@ def _hstack(arrays, join_type='exact', uniq_col_name='{col_name}_{table_name}',
             else:
                 out[out_name] = array[name][:n_rows]
 
-            _col_update_attrs_from(out[out_name], array[name])
-
     # If col_name_map supplied as a dict input, then update.
     if isinstance(_col_name_map, collections.Mapping):
         _col_name_map.update(col_name_map)
