@@ -6,35 +6,39 @@ from the installed astropy.  It makes use of the `pytest` testing framework.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from ..extern import six
-from ..extern.six.moves import cPickle as pickle
-
-import errno
-import shlex
-import sys
 import base64
-import zlib
+import errno
 import functools
-import multiprocessing
 import os
-import shutil
-import tempfile
+import sys
 import types
 import warnings
+import zlib
+
+from ..extern import six
+from ..extern.six.moves import cPickle as pickle
 
 try:
     # Import pkg_resources to prevent it from issuing warnings upon being
     # imported from within py.test.  See
     # https://github.com/astropy/astropy/pull/537 for a detailed explanation.
-    import pkg_resources
+    import pkg_resources  # pylint: disable=W0611
 except ImportError:
     pass
 
-from .. import test
-from ..utils.exceptions import (AstropyWarning,
-                                AstropyDeprecationWarning,
+from ..utils.exceptions import (AstropyDeprecationWarning,
                                 AstropyPendingDeprecationWarning)
-from ..config import configuration
+
+
+# For backward-compatibility with affiliated packages
+from .runner import TestRunner  # pylint: disable=W0611
+
+__all__ = ['raises', 'enable_deprecations_as_exceptions', 'remote_data',
+           'treat_deprecations_as_exceptions', 'catch_warnings',
+           'assert_follows_unicode_guidelines', 'quantity_allclose',
+           'assert_quantity_allclose', 'check_pickling_recovery',
+           'pickle_protocol', 'generic_recursive_equality_test']
+
 
 if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST') or '_pytest' in sys.modules:
     import pytest
@@ -96,196 +100,6 @@ _rewrite._write_pyc = _write_pyc_wrapper
 remote_data = pytest.mark.remote_data
 
 
-class TestRunner(object):
-    def __init__(self, base_path):
-        self.base_path = base_path
-
-    def run_tests(self, package=None, test_path=None, args=None, plugins=None,
-                  verbose=False, pastebin=None, remote_data=False, pep8=False,
-                  pdb=False, coverage=False, open_files=False, parallel=0,
-                  docs_path=None, skip_docs=False, repeat=None):
-        """
-        The docstring for this method lives in astropy/__init__.py:test
-        """
-
-        if coverage:
-            warnings.warn(
-                "The coverage option is ignored on run_tests, since it "
-                "can not be made to work in that context.  Use "
-                "'python setup.py test --coverage' instead.",
-                AstropyWarning)
-
-        all_args = []
-
-        if package is None:
-            package_path = self.base_path
-        else:
-            package_path = os.path.join(self.base_path,
-                                        package.replace('.', os.path.sep))
-
-            if not os.path.isdir(package_path):
-                raise ValueError('Package not found: {0}'.format(package))
-
-        if docs_path is not None and not skip_docs:
-            if package is not None:
-                docs_path = os.path.join(
-                    docs_path, package.replace('.', os.path.sep))
-            if not os.path.exists(docs_path):
-                warnings.warn(
-                    "Can not test .rst docs, since docs path "
-                    "({0}) does not exist.".format(docs_path))
-                docs_path = None
-
-        if test_path:
-            base, ext = os.path.splitext(test_path)
-
-            if ext in ('.rst', ''):
-                if docs_path is None:
-                    # This shouldn't happen from "python setup.py test"
-                    raise ValueError(
-                        "Can not test .rst files without a docs_path "
-                        "specified.")
-
-                abs_docs_path = os.path.abspath(docs_path)
-                abs_test_path = os.path.abspath(
-                    os.path.join(abs_docs_path, os.pardir, test_path))
-
-                common = os.path.commonprefix((abs_docs_path, abs_test_path))
-
-                if os.path.exists(abs_test_path) and common == abs_docs_path:
-                    # Since we aren't testing any Python files within
-                    # the astropy tree, we need to forcibly load the
-                    # astropy py.test plugins, and then turn on the
-                    # doctest_rst plugin.
-                    all_args.extend(['-p', 'astropy.tests.pytest_plugins',
-                                     '--doctest-rst'])
-                    test_path = abs_test_path
-
-            if not (os.path.isdir(test_path) or ext in ('.py', '.rst')):
-                raise ValueError("Test path must be a directory or a path to "
-                                 "a .py or .rst file")
-
-            all_args.append(test_path)
-        else:
-            all_args.append(package_path)
-            if docs_path is not None and not skip_docs:
-                all_args.extend([docs_path, '--doctest-rst'])
-
-        # add any additional args entered by the user
-        if args is not None:
-            all_args.extend(
-                shlex.split(args, posix=not sys.platform.startswith('win')))
-
-        # add verbosity flag
-        if verbose:
-            all_args.append('-v')
-
-        # turn on pastebin output
-        if pastebin is not None:
-            if pastebin in ['failed', 'all']:
-                all_args.append('--pastebin={0}'.format(pastebin))
-            else:
-                raise ValueError("pastebin should be 'failed' or 'all'")
-
-        # run @remote_data tests
-        if remote_data:
-            all_args.append('--remote-data')
-
-        if pep8:
-            try:
-                import pytest_pep8
-            except ImportError:
-                raise ImportError('PEP8 checking requires pytest-pep8 plugin: '
-                                  'http://pypi.python.org/pypi/pytest-pep8')
-            else:
-                all_args.extend(['--pep8', '-k', 'pep8'])
-
-        # activate post-mortem PDB for failing tests
-        if pdb:
-            all_args.append('--pdb')
-
-        # check for opened files after each test
-        if open_files:
-            if parallel != 0:
-                raise SystemError(
-                    "open file detection may not be used in conjunction with "
-                    "parallel testing.")
-
-            try:
-                import psutil
-            except ImportError:
-                raise SystemError(
-                    "open file detection requested, but psutil package "
-                    "is not installed.")
-
-            all_args.append('--open-files')
-
-            print("Checking for unclosed files")
-
-        if parallel != 0:
-            try:
-                import xdist
-            except ImportError:
-                raise ImportError(
-                    'Parallel testing requires the pytest-xdist plugin '
-                    'https://pypi.python.org/pypi/pytest-xdist')
-
-            try:
-                parallel = int(parallel)
-            except ValueError:
-                raise ValueError(
-                    "parallel must be an int, got {0}".format(parallel))
-
-            if parallel < 0:
-                parallel = multiprocessing.cpu_count()
-            all_args.extend(['-n', six.text_type(parallel)])
-
-        if repeat:
-            all_args.append('--repeat={0}'.format(repeat))
-
-        if six.PY2:
-            all_args = [x.encode('utf-8') for x in all_args]
-
-        # override the config locations to not make a new directory nor use
-        # existing cache or config
-        xdg_config_home = os.environ.get('XDG_CONFIG_HOME')
-        xdg_cache_home = os.environ.get('XDG_CACHE_HOME')
-        astropy_config = tempfile.mkdtemp('astropy_config')
-        astropy_cache = tempfile.mkdtemp('astropy_cache')
-        os.environ[str('XDG_CONFIG_HOME')] = str(astropy_config)
-        os.environ[str('XDG_CACHE_HOME')] = str(astropy_cache)
-        os.mkdir(os.path.join(os.environ['XDG_CONFIG_HOME'], 'astropy'))
-        os.mkdir(os.path.join(os.environ['XDG_CACHE_HOME'], 'astropy'))
-        # To fully force configuration reloading from a different file (in this
-        # case our default one in a temp directory), clear the config object
-        # cache.
-        configuration._cfgobjs.clear()
-
-        # This prevents cyclical import problems that make it
-        # impossible to test packages that define Table types on their
-        # own.
-        from ..table import Table
-
-        try:
-            result = pytest.main(args=all_args, plugins=plugins)
-        finally:
-            shutil.rmtree(os.environ['XDG_CONFIG_HOME'])
-            shutil.rmtree(os.environ['XDG_CACHE_HOME'])
-            if xdg_config_home is not None:
-                os.environ[str('XDG_CONFIG_HOME')] = xdg_config_home
-            else:
-                del os.environ['XDG_CONFIG_HOME']
-            if xdg_cache_home is not None:
-                os.environ[str('XDG_CACHE_HOME')] = xdg_cache_home
-            else:
-                del os.environ['XDG_CACHE_HOME']
-            configuration._cfgobjs.clear()
-
-        return result
-
-    run_tests.__doc__ = test.__doc__
-
-
 # This is for Python 2.x and 3.x compatibility.  distutils expects
 # options to all be byte strings on Python 2 and Unicode strings on
 # Python 3.
@@ -310,21 +124,26 @@ def _save_coverage(cov, result, rootdir, testing_path):
 
     # The coverage report includes the full path to the temporary
     # directory, so we replace all the paths with the true source
-    # path. This means that the coverage line-by-line report will only
-    # be correct for Python 2 code (since the Python 3 code will be
-    # different in the build directory from the source directory as
-    # long as 2to3 is needed). Therefore we only do this fix for
-    # Python 2.x.
-    if six.PY2:
-        d = cov.data
+    # path. Note that this will not work properly for packages that still
+    # rely on 2to3.
+    try:
+        # Coverage 4.0: _harvest_data has been renamed to get_data, the
+        # lines dict is private
+        cov.get_data()
+    except AttributeError:
+        # Coverage < 4.0
         cov._harvest_data()
-        for key in d.lines.keys():
-            new_path = os.path.relpath(
-                os.path.realpath(key),
-                os.path.realpath(testing_path))
-            new_path = os.path.abspath(
-                os.path.join(rootdir, new_path))
-            d.lines[new_path] = d.lines.pop(key)
+        lines = cov.data.lines
+    else:
+        lines = cov.data._lines
+
+    for key in list(lines.keys()):
+        new_path = os.path.relpath(
+            os.path.realpath(key),
+            os.path.realpath(testing_path))
+        new_path = os.path.abspath(
+            os.path.join(rootdir, new_path))
+        lines[new_path] = lines.pop(key)
 
     color_print('Saving coverage data in .coverage...', 'green')
     cov.save()
@@ -413,12 +232,12 @@ def treat_deprecations_as_exceptions():
     # before we turn the warnings into exceptions, we're golden.
     try:
         # A deprecated stdlib module used by py.test
-        import compiler
+        import compiler  # pylint: disable=W0611
     except ImportError:
         pass
 
     try:
-        import scipy
+        import scipy  # pylint: disable=W0611
     except ImportError:
         pass
 
@@ -432,15 +251,6 @@ def treat_deprecations_as_exceptions():
         warnings.filterwarnings("error", ".*", AstropyDeprecationWarning)
         warnings.filterwarnings("error", ".*", AstropyPendingDeprecationWarning)
 
-    if sys.version_info[:2] == (2, 6):
-        # py.test's warning.showwarning does not include the line argument
-        # on Python 2.6, so we need to explicitly ignore this warning.
-        warnings.filterwarnings(
-            "ignore",
-            r"functions overriding warnings\.showwarning\(\) must support "
-            r"the 'line' argument",
-            DeprecationWarning)
-
     if sys.version_info[:2] >= (3, 4):
         # py.test reads files with the 'U' flag, which is now
         # deprecated in Python 3.4.
@@ -481,7 +291,7 @@ class catch_warnings(warnings.catch_warnings):
     This completely blitzes any memory of any warnings that have
     appeared before so that all warnings will be caught and displayed.
 
-    *args is a set of warning classes to collect.  If no arguments are
+    ``*args`` is a set of warning classes to collect.  If no arguments are
     provided, all warnings are collected.
 
     Use as follows::
@@ -509,11 +319,49 @@ class catch_warnings(warnings.catch_warnings):
         treat_deprecations_as_exceptions()
 
 
+class ignore_warnings(catch_warnings):
+    """
+    This can be used either as a context manager or function decorator to
+    ignore all warnings that occur within a function or block of code.
+
+    An optional category option can be supplied to only ignore warnings of a
+    certain category or categories (if a list is provided).
+    """
+
+    def __init__(self, category=None):
+        super(ignore_warnings, self).__init__()
+
+        if isinstance(category, type) and issubclass(category, Warning):
+            self.category = [category]
+        else:
+            self.category = category
+
+    def __call__(self, func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+            # Originally this just reused self, but that doesn't work if the
+            # function is called more than once so we need to make a new
+            # context manager instance for each call
+            with self.__class__(category=self.category):
+                return func(*args, **kwargs)
+
+        return wrapper
+
+    def __enter__(self):
+        retval = super(ignore_warnings, self).__enter__()
+        if self.category is not None:
+            for category in self.category:
+                warnings.simplefilter('ignore', category)
+        else:
+            warnings.simplefilter('ignore')
+        return retval
+
+
 def assert_follows_unicode_guidelines(
         x, roundtrip=None):
     """
     Test that an object follows our Unicode policy.  See
-    "Unicode Policy" in the coding guidelines.
+    "Unicode guidelines" in the coding guidelines.
 
     Parameters
     ----------
