@@ -13,6 +13,7 @@ from ..extern.six.moves import filter
 
 import ast
 import doctest
+import datetime
 import fnmatch
 import imp
 import io
@@ -22,18 +23,19 @@ import os
 import re
 import sys
 import types
+from collections import OrderedDict
 
-from .helper import (
-    pytest, treat_deprecations_as_exceptions, enable_deprecations_as_exceptions)
+from ..config.paths import set_temp_config, set_temp_cache
+from .helper import pytest, treat_deprecations_as_exceptions, ignore_warnings
+from .helper import enable_deprecations_as_exceptions  # pylint: disable=W0611
 from .disable_internet import turn_off_internet, turn_on_internet
-from .output_checker import AstropyOutputChecker, FIX, FLOAT_CMP
-from ..utils import OrderedDict
+from .output_checker import AstropyOutputChecker, FIX
+from ..utils.argparse import writeable_directory
 from ..utils.introspection import resolve_name
 
-# Needed for Python 2.6 compatibility
 try:
     import importlib.machinery as importlib_machinery
-except ImportError:
+except ImportError:  # Python 2.7
     importlib_machinery = None
 
 # these pytest hooks allow us to mark tests and run the marked tests with
@@ -54,6 +56,20 @@ def pytest_addoption(parser):
     parser.addoption("--doctest-rst", action="store_true",
                      help="enable running doctests in .rst documentation")
 
+    parser.addoption("--config-dir", nargs='?', type=writeable_directory,
+                     help="specify directory for storing and retrieving the "
+                          "Astropy configuration during tests (default is "
+                          "to use a temporary directory created by the test "
+                          "runner); be aware that using an Astropy config "
+                          "file other than the default can cause some tests "
+                          "to fail unexpectedly")
+
+    parser.addoption("--cache-dir", nargs='?', type=writeable_directory,
+                     help="specify directory for storing and retrieving the "
+                          "Astropy cache during tests (default is "
+                          "to use a temporary directory created by the test "
+                          "runner)")
+
     parser.addini("doctest_plus", "enable running doctests with additional "
                   "features not found in the normal doctest plugin")
 
@@ -65,6 +81,27 @@ def pytest_addoption(parser):
                   "Run the doctests in the rst documentation",
                   default=False)
 
+    parser.addini("config_dir",
+                  "specify directory for storing and retrieving the "
+                  "Astropy configuration during tests (default is "
+                  "to use a temporary directory created by the test "
+                  "runner); be aware that using an Astropy config "
+                  "file other than the default can cause some tests "
+                  "to fail unexpectedly", default=None)
+
+    parser.addini("cache_dir",
+                  "specify directory for storing and retrieving the "
+                  "Astropy cache during tests (default is "
+                  "to use a temporary directory created by the test "
+                  "runner)", default=None)
+
+    parser.addini("open_files_ignore",
+                  "when used with the --open-files option, allows "
+                  "specifying names of files that may be ignored when "
+                  "left open between tests--files in this list are matched "
+                  "may be specified by their base name (ignoring their full "
+                  "path) or by absolute path", type="args", default=())
+
     parser.addoption('--repeat', action='store',
                      help='Number of times to repeat each test')
 
@@ -121,7 +158,10 @@ def pytest_configure(config):
         # handling __doctest_skip__) doesn't happen.
         def collect(self):
             if self.fspath.basename == "conftest.py":
-                module = self.config._conftest.importconftest(self.fspath)
+                try:
+                    module = self.config._conftest.importconftest(self.fspath)
+                except AttributeError:  # pytest >= 2.8.0
+                    module = self.config.pluginmanager._importconftest(self.fspath)
             else:
                 try:
                     module = self.fspath.pyimport()
@@ -151,8 +191,14 @@ def pytest_configure(config):
         def runtest(self):
             # satisfy `FixtureRequest` constructor...
             self.funcargs = {}
-            self._fixtureinfo = doctest_plugin.FuncFixtureInfo((), [], {})
-            fixture_request = doctest_plugin.FixtureRequest(self)
+            try:
+                self._fixtureinfo = doctest_plugin.FuncFixtureInfo((), [], {})
+                fixture_request = doctest_plugin.FixtureRequest(self)
+            except AttributeError:  # pytest >= 2.8.0
+                python_plugin = config.pluginmanager.getplugin('python')
+                self._fixtureinfo = python_plugin.FuncFixtureInfo((), [], {})
+                fixture_request = python_plugin.FixtureRequest(self)
+
             failed, tot = doctest.testfile(
                 str(self.fspath), module_relative=False,
                 optionflags=opts, parser=DocTestParserPlus(),
@@ -255,9 +301,6 @@ class DoctestPlus(object):
         # Directories to ignore when adding doctests
         self._ignore_paths = []
 
-        if run_rst_doctests and six.PY3:
-            self._run_rst_doctests = False
-
     def pytest_ignore_collect(self, path, config):
         """Skip paths that match any of the doctest_norecursedirs patterns."""
 
@@ -315,10 +358,10 @@ class DoctestPlus(object):
             return self._doctest_module_item_cls(path, parent)
         elif self._run_rst_doctests and path.ext == '.rst':
             # Ignore generated .rst files
-            parts = bytes(path).split(os.path.sep)
-            if (path.basename.startswith(b'_') or
-                any(x.startswith(b'_') for x in parts) or
-                any(x == b'api' for x in parts)):
+            parts = str(path).split(os.path.sep)
+            if (path.basename.startswith('_') or
+                    any(x.startswith('_') for x in parts) or
+                    any(x == 'api' for x in parts)):
                 return None
 
             # TODO: Get better names on these items when they are
@@ -415,6 +458,22 @@ def _get_open_file_list():
 
 
 def pytest_runtest_setup(item):
+    config_dir = item.config.getini('config_dir')
+    cache_dir = item.config.getini('cache_dir')
+
+    # Command-line options can override, however
+    config_dir = item.config.getoption('config_dir') or config_dir
+    cache_dir = item.config.getoption('cache_dir') or cache_dir
+
+    # We can't really use context managers directly in py.test (although
+    # py.test 2.7 adds the capability), so this may look a bit hacky
+    if config_dir:
+        item.set_temp_config = set_temp_config(config_dir)
+        item.set_temp_config.__enter__()
+    if cache_dir:
+        item.set_temp_cache = set_temp_cache(cache_dir)
+        item.set_temp_cache.__enter__()
+
     # Store a list of the currently opened files so we can compare
     # against them when the test is done.
     if item.config.getvalue('open_files'):
@@ -426,6 +485,11 @@ def pytest_runtest_setup(item):
 
 
 def pytest_runtest_teardown(item, nextitem):
+    if hasattr(item, 'set_temp_cache'):
+        item.set_temp_cache.__exit__()
+    if hasattr(item, 'set_temp_config'):
+        item.set_temp_config.__exit__()
+
     # a "skipped" test will not have been called with
     # pytest_runtest_setup, so therefore won't have an
     # "open_files" member
@@ -445,13 +509,21 @@ def pytest_runtest_teardown(item, nextitem):
         return
 
     not_closed = set()
+    open_files_ignore = item.config.getini('open_files_ignore')
     for filename in open_files:
-        # astropy.log files are allowed to continue to exist
-        # between test runs
-        if (os.path.basename(filename) == 'astropy.log' or
-                filename == '/etc/hosts'):
-            # /etc/hosts exclusion is a limited backport of
-            # https://github.com/astropy/astropy/pull/3954
+        ignore = False
+
+        for ignored in open_files_ignore:
+            if not os.path.isabs(ignored):
+                if os.path.basename(filename) == ignored:
+                    ignore = True
+                    break
+            else:
+                if filename == ignored:
+                    ignore = True
+                    break
+
+        if ignore:
             continue
 
         if filename not in start_open_files:
@@ -467,7 +539,8 @@ def pytest_runtest_teardown(item, nextitem):
 PYTEST_HEADER_MODULES = OrderedDict([('Numpy', 'numpy'),
                                      ('Scipy', 'scipy'),
                                      ('Matplotlib', 'matplotlib'),
-                                     ('h5py', 'h5py')])
+                                     ('h5py', 'h5py'),
+                                     ('Pandas', 'pandas')])
 
 # This always returns with Astropy's version
 from .. import __version__
@@ -477,7 +550,10 @@ TESTED_VERSIONS = OrderedDict([('Astropy', __version__)])
 
 def pytest_report_header(config):
 
-    stdoutencoding = getattr(sys.stdout, 'encoding') or 'ascii'
+    try:
+        stdoutencoding = sys.stdout.encoding or 'ascii'
+    except AttributeError:
+        stdoutencoding = 'ascii'
 
     if six.PY2:
         args = [x.decode('utf-8') for x in config.args]
@@ -494,7 +570,21 @@ def pytest_report_header(config):
         s = "\nRunning tests with Astropy version {0}.\n".format(
             TESTED_VERSIONS['Astropy'])
 
-    s += "Running tests in {0}.\n\n".format(" ".join(args))
+    # Per https://github.com/astropy/astropy/pull/4204, strip the rootdir from
+    # each directory argument
+    if hasattr(config, 'rootdir'):
+        rootdir = str(config.rootdir)
+        if not rootdir.endswith(os.sep):
+            rootdir += os.sep
+
+        dirs = [arg[len(rootdir):] if arg.startswith(rootdir) else arg
+                for arg in args]
+    else:
+        dirs = args
+
+    s += "Running tests in {0}.\n\n".format(" ".join(dirs))
+
+    s += "Date: {0}\n\n".format(datetime.datetime.now().isoformat()[:19])
 
     from platform import platform
     plat = platform()
@@ -519,7 +609,8 @@ def pytest_report_header(config):
 
     for module_display, module_name in six.iteritems(PYTEST_HEADER_MODULES):
         try:
-            module = resolve_name(module_name)
+            with ignore_warnings(DeprecationWarning):
+                module = resolve_name(module_name)
         except ImportError:
             s += "{0}: not available\n".format(module_display)
         else:
@@ -537,9 +628,6 @@ def pytest_report_header(config):
     if opts:
         s += "Using Astropy options: {0}.\n".format(" ".join(opts))
 
-    if six.PY3 and (config.getini('doctest_rst') or config.option.doctest_rst):
-        s += "Running doctests in .rst files is not supported on Python 3.x\n"
-
     if not six.PY3:
         s = s.encode(stdoutencoding, 'replace')
 
@@ -688,6 +776,7 @@ class NoUnicodeLiteralsModule(ModifiedModule):
                 return node
         return NonAsciiLiteral().visit(tree)
 
+
 def pytest_unconfigure():
     """
     Cleanup post-testing
