@@ -345,6 +345,8 @@ def test_spectraldensity4():
     flux_flam = [3.9135e-14, 4.0209e-14, 3.9169e-14]
     flux_fnu = [3.20735792e-25, 3.29903646e-25, 3.21727226e-25]
     flux_jy = [3.20735792e-2, 3.29903646e-2, 3.21727226e-2]
+    flux_stmag = [12.41858665, 12.38919182, 12.41764379]
+    flux_abmag = [12.63463143, 12.60403221, 12.63128047]
 
     # PHOTLAM <--> FLAM
     assert_allclose(photlam.to(
@@ -382,13 +384,29 @@ def test_spectraldensity4():
     assert_allclose(flam.to(
         photnu, flux_flam, u.spectral_density(wave)), flux_photnu, rtol=1e-6)
 
+    # PHOTLAM <--> STMAG
+    assert_allclose(photlam.to(
+        u.STmag, flux_photlam, u.spectral_density(wave)), flux_stmag, rtol=1e-6)
+    assert_allclose(u.STmag.to(
+        photlam, flux_stmag, u.spectral_density(wave)), flux_photlam, rtol=1e-6)
+
+    # PHOTLAM <--> ABMAG
+    assert_allclose(photlam.to(
+        u.ABmag, flux_photlam, u.spectral_density(wave)), flux_abmag, rtol=1e-6)
+    assert_allclose(u.ABmag.to(
+        photlam, flux_abmag, u.spectral_density(wave)), flux_photlam, rtol=1e-6)
+
 
 def test_equivalent_units():
-    units = u.g.find_equivalent_units()
-    units_set = set(units)
-    match = set(
-        [u.M_e, u.M_p, u.g, u.kg, u.solMass, u.t, u.u])
-    assert units_set == match
+    from .. import imperial
+    with u.add_enabled_units(imperial):
+        units = u.g.find_equivalent_units()
+        units_set = set(units)
+        match = set(
+            [u.M_e, u.M_p, u.g, u.kg, u.solMass, u.t, u.u, u.M_earth,
+             u.M_jup, imperial.oz, imperial.lb, imperial.st, imperial.ton,
+             imperial.slug])
+        assert units_set == match
 
     r = repr(units)
     assert r.count('\n') == len(units) + 2
@@ -398,7 +416,8 @@ def test_equivalent_units2():
     units = set(u.Hz.find_equivalent_units(u.spectral()))
     match = set(
         [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr,
-         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k])
+         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k, u.earthRad,
+         u.jupiterRad])
     assert units == match
 
     from .. import imperial
@@ -406,16 +425,17 @@ def test_equivalent_units2():
         units = set(u.Hz.find_equivalent_units(u.spectral()))
         match = set(
             [u.AU, u.Angstrom, imperial.BTU, u.Hz, u.J, u.Ry,
-             imperial.cal, u.cm, u.eV, u.erg, imperial.ft,
+             imperial.cal, u.cm, u.eV, u.erg, imperial.ft, imperial.fur,
              imperial.inch, imperial.kcal, u.lyr, u.m, imperial.mi,
-             u.micron, u.pc, u.solRad, imperial.yd, u.Bq, u.Ci,
-             imperial.nmi, u.k])
+             imperial.mil, u.micron, u.pc, u.solRad, imperial.yd, u.Bq, u.Ci,
+             imperial.nmi, u.k, u.earthRad, u.jupiterRad])
         assert units == match
 
     units = set(u.Hz.find_equivalent_units(u.spectral()))
     match = set(
         [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr,
-         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k])
+         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k, u.earthRad,
+         u.jupiterRad])
     assert units == match
 
 
