@@ -12,8 +12,8 @@ import itertools
 import numpy as np
 from numpy.testing.utils import assert_allclose
 
-from ...tests.helper import pytest
-from ... import units as u
+from ...tests.helper import pytest, assert_quantity_allclose
+from ... import units as u, constants as c
 
 lu_units = [u.dex, u.mag, u.decibel]
 
@@ -71,6 +71,16 @@ class TestLogUnitCreation(object):
             lu_cls(physical_unit, u.m)
 
 
+def test_predefined_magnitudes():
+    assert_quantity_allclose((-21.1*u.STmag).physical,
+                             1.*u.erg/u.cm**2/u.s/u.AA)
+    assert_quantity_allclose((-48.6*u.ABmag).physical,
+                             1.*u.erg/u.cm**2/u.s/u.Hz)
+    assert_quantity_allclose((0*u.M_bol).physical, c.L_bol0)
+    assert_quantity_allclose((0*u.m_bol).physical,
+                             c.L_bol0/(4.*np.pi*(10.*c.pc)**2))
+
+
 class TestLogUnitStrings(object):
 
     def test_str(self):
@@ -135,7 +145,7 @@ class TestLogUnitConversion(object):
     def test_container_unit_conversion(self, lu_unit):
         """Check that conversion to logarithmic units (u.mag, u.dB, u.dex)
         is only possible when the physical unit is dimensionless."""
-        values = np.linspace(0.,10.,6.)
+        values = np.linspace(0., 10., 6)
         lu1 = lu_unit(u.dimensionless_unscaled)
         assert lu1.is_equivalent(lu1.function_unit)
         assert_allclose(lu1.to(lu1.function_unit, values), values)
@@ -151,7 +161,7 @@ class TestLogUnitConversion(object):
     def test_subclass_conversion(self, flu_unit, tlu_unit, physical_unit):
         """Check various LogUnit subclasses are equivalent and convertible
         to each other if they correspond to equivalent physical units."""
-        values = np.linspace(0.,10.,6.)
+        values = np.linspace(0., 10., 6)
         flu = flu_unit(physical_unit)
 
         tlu = tlu_unit(physical_unit)
