@@ -18,7 +18,7 @@ from numpy.testing import (
 
 from ...tests.helper import raises, catch_warnings, pytest
 from ... import wcs
-from .. import _wcs
+from .. import _wcs  # pylint: disable=W0611
 from ...utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
 from ...utils.misc import NumpyRNGContext
@@ -914,3 +914,73 @@ def test_no_truncate_using_compare():
     w.wcs.set()
     w2 = wcs.WCS(w.to_header())
     w.wcs.compare(w2.wcs)
+
+
+def test_passing_ImageHDU():
+    """
+    Passing ImageHDU or PrimaryHDU and comparing it with
+    wcs initialized from header. For #4493.
+    """
+    path = get_pkg_data_filename('data/validate.fits')
+    hdulist = fits.open(path)
+    wcs_hdu = wcs.WCS(hdulist[0])
+    wcs_header = wcs.WCS(hdulist[0].header)
+    assert wcs_hdu.wcs.compare(wcs_header.wcs)
+    wcs_hdu = wcs.WCS(hdulist[1])
+    wcs_header = wcs.WCS(hdulist[1].header)
+    assert wcs_hdu.wcs.compare(wcs_header.wcs)
+
+
+def inconsistent_sip():
+    """
+    Test for #4814
+    """
+    hdr = get_pkg_data_contents("data/sip-broken.hdr")
+    w = wcs.WCS(hdr)
+    newhdr = w.to_header(relax=None)
+    # CTYPE should not include "-SIP" if relax is None
+    assert(all([ctyp[-4 :] != '-SIP' for ctyp in self.wcs.ctype]))
+    newhdr = w.to_header(relax=False)
+    assert('A_0_2' not in newhdr)
+    # CTYPE should not include "-SIP" if relax is False
+    assert(all([ctyp[-4 :] != '-SIP' for ctyp in self.wcs.ctype]))
+    newhdr = w.to_header(key="C")
+    assert('A_0_2' not in newhdr)
+    # Test writing header with a different key
+    assert(all([ctyp[-4 :] != '-SIP' for ctyp in self.wcs.ctype]))
+    newhdr = w.to_header(key=" ")
+    # Test writing a primary WCS to header
+    assert(all([ctyp[-4 :] != '-SIP' for ctyp in self.wcs.ctype]))
+    # Test that "-SIP" is kept into CTYPE if relax=True and
+    # "-SIP" was in the original header
+    newhdr = w.to_header(relax=True)
+    assert(all([ctyp[-4 :] == '-SIP' for ctyp in self.wcs.ctype]))
+    assert('A_0_2' in newhdr)
+    # Test that SIP coefficients are also written out.
+    wtest = WCS(newhdr)
+    assert wtest.sip is not None
+    ########## broken header ###########
+    # Test that "-SIP" is added to CTYPE if relax=True and
+    # "-SIP" was not in the original header
+    hdr['CTYPE1'] = 'RA---TAN'
+    w = WCS(hdr)
+    newhdr = w.to_header(relax=True)
+    assert(all([ctyp[-4 :] == '-SIP' for ctyp in self.wcs.ctype]))
+    del hdr['CTYPE2']
+    newhdr = w.to_header(relax=True)
+    assert(all([ctyp[-4 :] == '-SIP' for ctyp in self.wcs.ctype]))
+    w = wcs.WCS()
+    newhdr = w.to_header()
+    assert('CTYPE1' not in newhdr)
+
+def test_bounds_check():
+    """Test for #4957"""
+    w = wcs.WCS(naxis=2)
+    w.wcs.ctype = ["RA---CAR","DEC--CAR"]
+    w.wcs.cdelt = [10,10]
+    w.wcs.crval = [-90,90]
+    w.wcs.crpix = [1,1]
+    w.wcs.bounds_check(False, False)
+    ra, dec = w.wcs_pix2world(300,0,0)
+    assert_allclose(ra, -180)
+    assert_allclose(dec, -30)
